DECLARE Sub DeInit ()
DECLARE Sub Initialize ()
DECLARE sub Initialize_GameConfig ()
DECLARE Sub Initialize_GLFW ()
DECLARE Sub Initialize_OpenGL ()
DECLARE Sub Initialize_Physics ()
DECLARE sub Initialize_WindowResizeCallback GLFWCALL (w as integer, h as integer)

DECLARE sub Set_LightVars (lightNum as integer, lightVar as integer, a as single, b as single, c as single)
DECLARE sub Upd_LightPos (lightNum as integer, x as single, y as single, z as single)





type OpenGLPropertiesType

	SCR_WIDTH		as integer
	SCR_HEIGHT		as integer
	SCR_BITS		as integer
	SCR_FULLSCREEN	as integer
	SCR_BORDER		as integer
	WINDOW_RESIZED	as integer
	
end type


dim shared OpenGLProperties as OpenGLPropertiesType



const VIDEO_FULLSCREEN = GLFW_FULLSCREEN, VIDEO_WINDOWED = GLFW_WINDOW
CONST INPUT_KEYBOARD = 0, INPUT_GAMECONTROLLER = 1

type configVarType
    xres    			as integer
    yres    			as integer
    nBits   			as byte
    fullScreen  		as byte
    inputType   		as byte
    axisMin(5)  		as single
    axisMax(5)  		as single
    axisCenter(5) 		as single
    joystick    		as byte
    control(12)  		as byte
    displayOption(23)	as byte
	lastJoy				as byte
end type

'#define OPTION_OUTLINES 		0
'#define OPTION_LINESMOOTHING 	1
'#DEFINE OPTION_ENGINEWIND 		2
'#DEFINE OPTION_DOME 			3
'#DEFINE OPTION_BUILDINGS 		4
'#DEFINE OPTION_BLOOM 			5
'#DEFINE OPTION_HUDHOUSING 		6
'#DEFINE OPTION_FSAA 			7
'#DEFINE OPTION_ENVMAP 			8
dim shared configVars as configVarType, joyAxis(5)  as single




sub DeInit ()

	
	dim iter as integer
	
	for iter = 0 to ModelDBLength - 1
		UnloadMesh ModelDB[iter].modelHandle
	next
	
	glfwTerminate
	
	
end sub






sub Initialize ()


	Initialize_GameConfig
	Initialize_GLFW
	Initialize_OpenGL
	Initialize_FontDB
	Initialize_Physics
	
	

end sub




sub Initialize_GameConfig ()
    
	dim axis as integer, control as integer, displayoption as integer, lastjoy as integer, joy as integer
	dim success as integer, buttons as integer, x as single, y as single
	
    open "configVars.dat" for input as #1
        input #1, configVars.xres
        input #1, configVars.yres
        input #1, configVars.nBits
        input #1, configVars.fullScreen
		input #1, PhaseMenuGlobals.winOptions.musicOn
		input #1, PhaseMenuGlobals.winOptions.sfxOn
		input #1, PhaseMenuGlobals.winOptions.musicVol
		input #1, PhaseMenuGlobals.winOptions.sfxVol
		input #1, PhaseMenuGlobals.winOptions.sensitivity
		input #1, PhaseMenuGlobals.winOptions.invertMouse
		input #1, PhaseMenuGlobals.winOptions.difficulty
    close #1
    


    OpenGLProperties.SCR_BITS = configVars.nBits
	OpenGLProperties.SCR_WIDTH = configVars.xres
	OpenGLProperties.SCR_HEIGHT = configVars.yres
	if configVars.fullScreen = 1 then OpenGLProperties.SCR_FULLSCREEN = GLFW_FULLSCREEN
	if configVars.fullScreen = 0 then OpenGLProperties.SCR_FULLSCREEN = GLFW_WINDOW
	
    
end sub



sub Initialize_GLFW ()


	glfwInit
	
	'----------------
	' setup window
	'----------------
	dim as integer mode
	
	'mode = 1
	
	
	
	
	'if mode = 1 then
	'	OpenGLProperties.SCR_WIDTH = 1280
	'	OpenGLProperties.SCR_HEIGHT = 800
	'	OpenGLProperties.SCR_FULLSCREEN = GLFW_WINDOW'FULLSCREEN
	'else
	'	OpenGLProperties.SCR_WIDTH = 1280
	'	OpenGLProperties.SCR_HEIGHT = 800
	'	OpenGLProperties.SCR_FULLSCREEN = GLFW_FULLSCREEN
	'end if
	
	dim ret as integer
	
	if OpenGLProperties.SCR_BITS = 16 then ret = glfwOpenWindow (OpenGLProperties.SCR_WIDTH, OpenGLProperties.SCR_HEIGHT, 5, 6, 5, 0, 24, 8, OpenGLProperties.SCR_FULLSCREEN)
	if OpenGLProperties.SCR_BITS = 32 then ret = glfwOpenWindow (OpenGLProperties.SCR_WIDTH, OpenGLProperties.SCR_HEIGHT, 8, 8, 8, 8, 24, 8, OpenGLProperties.SCR_FULLSCREEN)
	
	if ret = 0 then
		glfwTerminate
		end 1
	end if
	
	'glfwSetWindowTitle "Prompt Critical - Phase Sequence Editor"
	
	'OpenGLProperties.SCR_BORDER = (OpenGLProperties.SCR_HEIGHT - OpenGLProperties.SCR_WIDTH * .625) / 4
	
	glViewport 0, 0, OpenGLProperties.SCR_WIDTH, OpenGLProperties.SCR_HEIGHT
	'glViewport 0, OpenGLProperties.SCR_BORDER, OpenGLProperties.SCR_WIDTH, OpenGLProperties.SCR_HEIGHT - OpenGLProperties.SCR_BORDER * 2
	
	
	'if OpenGLProperties.SCR_FULLSCREEN = GLFW_FULLSCREEN then glfwDisable GLFW_MOUSE_CURSOR
	glfwEnable GLFW_MOUSE_CURSOR
	
	
	dim resizefun as GLFWwindowsizefun
	
	resizefun = @Initialize_WindowResizeCallback
	
	glfwSetWindowSizeCallback resizefun
	
	font_loadfont "vera"
	

end sub




sub Initialize_OpenGL ()


	'----------------
	' setup projection ratio
	'----------------
	glMatrixMode GL_PROJECTION
    glLoadIdentity
    
	dim FOVy as integer, aspect as single, znear as single, zfar as single
    FOVy = 40
    aspect = OpenGLProperties.SCR_WIDTH / OpenGLProperties.SCR_HEIGHT
	'aspect = OpenGLProperties.SCR_WIDTH / (OpenGLProperties.SCR_HEIGHT - OpenGLProperties.SCR_BORDER * 2)
    znear = .1
    zfar = 1000
    
    gluPerspective FOVy, aspect, znear, zfar
    
	glMatrixMode GL_MODELVIEW
    glLoadIdentity
	
	'-----------------
	' setup misc opengl display options
	'-----------------
	
	glShadeModel GL_SMOOTH
    
    glClearDepth 1.0
    glEnable GL_DEPTH_TEST
    glDepthFunc GL_LEQUAL
    glEnable GL_COLOR_MATERIAL
    glHint GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST
	
	Set_LightVars 0, GL_AMBIENT, .1, .1, .1
	Set_LightVars 0, GL_DIFFUSE, .9, .9, 1
	Set_LightVars 0, GL_SPECULAR, .9, .9, 1
	Set_LightVars 0, GL_POSITION, 3, 3, 3
	glMaterialf GL_FRONT, GL_SHININESS, 75
	
	
	glEnable GL_LIGHT0
    glEnable GL_LIGHTING
	glColorMaterial GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE
	
	glCullFace GL_BACK
    glEnable GL_CULL_FACE
    glHint GL_LINE_SMOOTH_HINT, GL_NICEST
	glHint GL_FOG_HINT, GL_FASTEST
    glBlendFunc GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA
    glEnable GL_BLEND
    glPolygonMode GL_FRONT, GL_FILL
	
	dim as rgbalpha fogcolor
	fogColor.r = .71
	fogColor.g = .68
	fogColor.b = .63
	glFogfv GL_FOG_COLOR, @fogColor.r
	glFogi GL_FOG_MODE, GL_LINEAR
    glHint GL_FOG_HINT, GL_DONT_CARE
    
    '
	
	'glEnable GL_MULTISAMPLE_ARB
	glEnable GL_LINE_SMOOTH



end sub



sub Initialize_Physics ()

	Physics_Initialize
	
end sub



sub Set_LightVars (lightNum as integer, lightVar as integer, a as single, b as single, c as single)

	dim tempVar(3) as single
	
	tempVar(0) = a
	tempVar(1) = b
	tempVar(2) = c
	tempVar(3) = 0
	
	glLightfv GL_LIGHT0 + lightNum, lightVar, @tempVar(0)
    
end sub




sub Upd_LightPos (lightNum as integer, x as single, y as single, z as single)

	dim tempVar(3) as single
	
	glGetFloatv GL_LIGHT0 + lightNum, @tempVar(0)
	
	tempVar(0) += x
	tempVar(1) += y
	tempVar(2) += z
	tempVar(3) = 1

	glLightfv GL_LIGHT0 + lightNum, GL_POSITION, @tempVar(0)

end sub



sub Initialize_WindowResizeCallback GLFWCALL (w as integer, h as integer)


	OpenGLProperties.SCR_WIDTH = w
	OpenGLProperties.SCR_HEIGHT = h
	
	glMatrixMode GL_PROJECTION
    glLoadIdentity
    
	OpenGLProperties.SCR_BORDER = (OpenGLProperties.SCR_HEIGHT - OpenGLProperties.SCR_WIDTH * .5) / 2
	
	if OpenGLProperties.SCR_BORDER < 0 then OpenGLProperties.SCR_BORDER = 0
	
	glViewport 0, 0, OpenGLProperties.SCR_WIDTH, OpenGLProperties.SCR_HEIGHT
	'glViewport 0, OpenGLProperties.SCR_BORDER, OpenGLProperties.SCR_WIDTH, OpenGLProperties.SCR_HEIGHT - OpenGLProperties.SCR_BORDER * 2
	
	dim FOVy as integer, aspect as single, znear as single, zfar as single
    FOVy = 40
    aspect = OpenGLProperties.SCR_WIDTH / OpenGLProperties.SCR_HEIGHT
	'aspect = OpenGLProperties.SCR_WIDTH / (OpenGLProperties.SCR_HEIGHT - OpenGLProperties.SCR_BORDER * 2)
    znear = .1
    zfar = 1000
    
    gluPerspective FOVy, aspect, znear, zfar
    
	glMatrixMode GL_MODELVIEW
	
	OpenGLProperties.WINDOW_RESIZED = 1
   
	
end sub


