
Sub Get_Inverse_Matrix(  Byref Inverse As Single Ptr , Byref m As Single Ptr )
 
	Inverse[0]  = m[0]: Inverse[1] = m[4]: Inverse[2]  = m[8]
	Inverse[4]  = m[1]: Inverse[5] = m[5]: Inverse[6]  = m[9]
	Inverse[8]  = m[2]: Inverse[9] = m[6]: Inverse[10] = m[10]
	Inverse[3]  = 0.0f: Inverse[7] = 0.0f: Inverse[11] = 0.0f
	Inverse[15] = 1.0f
 
	Inverse[12] = -(m[12] * m[0]) - (m[13] * m[1]) - (m[14] * m[2])
	Inverse[13] = -(m[12] * m[4]) - (m[13] * m[5]) - (m[14] * m[6])
	Inverse[14] = -(m[12] * m[8]) - (m[13] * m[9]) - (m[14] * m[10])
 
End Sub





function math_dot (byref v1 as vector4d, byref v2 as vector4d) as single


	dim as single retVal
	
	retVal = v1.x * v2.x
	retVal+= v1.y * v2.y
	retVal+= v1.z * v2.z
	retVal+= v1.w * v2.w
	
	return retVal
	
end function


sub math_normalize (byref v1 as vector4d)

	dim as single d
	
	d = sqr(v1.x * v1.x + v1.y * v1.y + v1.z * v1.z)
	if d <> 0 then
		v1.x /= d
		v1.y /= d
		v1.z /= d
	else
		v1.x = 0
		v1.y = 0
		v1.z = 0
	end if
	
end sub


sub math_normalize3f (byref v1 as vector3d)

	dim as single d
	
	d = sqr(v1.x * v1.x + v1.y * v1.y + v1.z * v1.z)
	
	if d <> 0 then
		v1.x /= d
		v1.y /= d
		v1.z /= d
	else
		v1.x = 0
		v1.y = 0
		v1.z = 0
	end if
	
end sub


sub math_sphere2cartesian (radius as single, height as single, byref x as single, byref y as single, byref z as single, an1 as single, an2 as single)

	dim f as single
    
    f = radius * sin(rad * an1)
    y = height * cos(rad * an1)
    z = f * cos(rad * an2)
    x = f * sin(rad * an2)
    
    
end sub


sub math_cartesian2sphere (byval x0 as single, byval y0 as single, byval z0 as single, byref an1 as single, byref an2 as single)

	dim as single r0, ThetaX, ThetaY, xR
	
	r0 = sqr(x0 * x0 + y0 * y0 + z0 * z0)
	
	an1 = 0
	an2 = 0
	if r0 <> 0 then 
		ThetaX = asin(y0 / r0)
		xR = cos(ThetaX) * r0
		
		ThetaY = asin(z0 / xR)
		if ThetaY > 180 or ThetaY < -180 then ThetaY = 0
		
		an1 = 90 - ThetaX * 180 / 3.14159
		an2 = (90 - ThetaY * 180 / 3.14159) * sgn(x0)
	end if

end sub	


sub math_cartesian2sphereB (byval x0 as single, byval y0 as single, byval z0 as single, byref an1 as single, byref an2 as single)

	dim as single r0, ThetaX, ThetaY, xR
	
	r0 = sqr(x0 * x0 + y0 * y0 + z0 * z0)
	
	an1 = 0
	an2 = 0
	if r0 <> 0 then 
		
		ThetaX = asin(y0 / r0)
		xR = cos(ThetaX) * r0
		
		
		ThetaY = asin(z0 / xR)
		if ThetaY > 180 or ThetaY < -180 then ThetaY = 0
		
		if z0 < 0 then 
			an1 = ThetaX * 180 / 3.14159
		else
			an1 = 180 - ThetaX * 180 / 3.14159
		end if
		
		if x0 < 0 then
			an2 = ThetaY * 180 / 3.14159
		else
			an2 = 180 - ThetaY * 180 / 3.14159
		end if
	end if

end sub	


function math_cross (v1 as Vector3d, v2 as Vector3d) as Vector3D
    
	dim ax1 as single, ay1 as single, az1 as single
	dim ax2 as single, ay2 as single, az2 as single
	dim ax3 as single, ay3 as single, az3 as single
	dim d as single, d2 as single, d3 as single
	dim as single nx, ny, nz
	
	dim v3 as Vector3D
	
    ax2 = v1.x
    ay2 = v1.y
    az2 = v1.z
    
    ax3 = v2.x
    ay3 = v2.y
    az3 = v2.z
    
    d2 = sqr(ax2 ^2 + ay2 ^2 + az2 ^2)
    d3 = sqr(ax3 ^2 + ay3 ^2 + az3 ^2)
    
    ax2 = ax2 / d2
    ay2 = ay2 / d2
    az2 = az2 / d2
    
    ax3 = ax3 / d3
    ay3 = ay3 / d3
    az3 = az3 / d3
    
    nx = ay2 * az3 - az2 * ay3
    ny = az2 * ax3 - ax2 * az3
    nz = ax2 * ay3 - ay2 * ax3
    
    d = sqr(nx * nx + ny * ny + nz * nz)
    v3.x = nx / d
    v3.y = ny / d
    v3.z = nz / d
	
	return v3
    
end function




Sub Matrix_LoadIdentity ( byval matrix as single ptr )

	dim i as integer
	
	for i = 0 to 15
		matrix[i] = 0
	next
	
	matrix[0] = 1
	matrix[5] = 1
	matrix[10] = 1
	matrix[15] = 1
	
	
end sub





sub Matrix_Multiply ( byval matrixA as single ptr, byval matrixB as single ptr )


	dim as integer columns, rows
	dim as integer col, row
	dim as single sum
	
	dim as single matrixC(15)
	
	columns = 4
	rows = 4
	
	
	for col = 0 to 3
		for row = 0 to 3
			MatrixC(col * 4 + row) = MatrixB[row + 0] * MatrixA[col * 4 + 0] + MatrixB[row + 4] * MatrixA[col * 4 + 1] + MatrixB[row + 8] * MatrixA[col * 4 + 2] + MatrixB[row + 12] * MatrixA[col * 4 + 3]
		next
	next
	
	
	
	
	dim i as integer
	
	for i = 0 to 15
		MatrixA[i] = MatrixC(i)
	next
	
	


end sub





sub Matrix_Multiply_Point ( byval matrixA as single ptr, byref matrixB as vector4d)

	dim matrixC as vector4d
	
	
	matrixC.x = matrixA[0] * matrixB.x + matrixA[4] * matrixB.y + matrixA[8] * matrixB.z + matrixA[12] * matrixB.w
	matrixC.y = matrixA[1] * matrixB.x + matrixA[5] * matrixB.y + matrixA[9] * matrixB.z + matrixA[13] * matrixB.w
	matrixC.z = matrixA[2] * matrixB.x + matrixA[6] * matrixB.y + matrixA[10] * matrixB.z + matrixA[14] * matrixB.w
	'matrixC.w = matrixA[3] * matrixB.x + matrixA[7] * matrixB.y + matrixA[11] * matrixB.z + matrixA[15] * matrixB.w
	matrixC.w = matrixB.w
	
	matrixB = matrixC

end sub





sub Matrix_Rotate ( byval matrix as single ptr, byval angle as single, byval x as single, byval y as single, byval z as single )


	dim as single rotMatrixX(15), rotMatrixY(15), rotMatrixZ(15)
	
	dim as single xAngle, yAngle, zAngle
	
	if x <> 0 then
		xAngle = angle * x * rad
		Matrix_LoadIdentity @rotMatrixX(0)
		rotMatrixX(5) = cos(xAngle)
		rotMatrixX(6) = -sin(xAngle)
		rotMatrixX(9) = sin(xAngle)
		rotMatrixX(10) = cos(xAngle)
		Matrix_Multiply matrix, @rotMatrixX(0)
	end if
	
	if y <> 0 then
		yAngle = angle * y * rad
		Matrix_LoadIdentity @rotMatrixY(0)
		rotMatrixY(0) = cos(yAngle)
		rotMatrixY(2) = sin(yAngle)
		rotMatrixY(8) = -sin(yAngle)
		rotMatrixY(10) = cos(yAngle)
		Matrix_Multiply matrix, @rotMatrixY(0)
	end if
	
	if z <> 0 then
		zAngle = angle * z * rad
		Matrix_LoadIdentity @rotMatrixZ(0)
		rotMatrixZ(0) = cos(zAngle)
		rotMatrixZ(1) = -sin(zAngle)
		rotMatrixZ(4) = sin(zAngle)
		rotMatrixZ(5) = cos(zAngle)
		Matrix_Multiply matrix, @rotMatrixZ(0)
	end if
	
	'Matrix_Mul_Matrix matrix, @rotMatrixZ(0)
	
	
end sub




sub Matrix_Translate ( byval matrix as single ptr, byval x as single, byval y as single, byval z as single )


	matrix[12] += x
	matrix[13] += y
	matrix[14] += z
	

end sub


