DECLARE sub Explosion_DeInit ()
DECLARE sub Explosion_Initialize ()
DECLARE sub Explosion_LaunchApp ()
DECLARE sub Explosion_MainLoop ()
DECLARE sub Explosion_ProcessInputs ()
DECLARE sub Explosion_Render ()
DECLARE sub Explosion_Shutdown ()


'--------------------
'  Globals - globals for this module are stored in this type then shared
'--------------------

type ExplostionType

	startTime	as single
	
end type


type GradSliderType
	position	as Vector3D
	angle		as Vector3D
	velocity	as Vector3D
	velocityMag	as single
end type


type ExplosionGlobalsType


	initialized	as byte
	exitCode	as byte

	Explosion	as ExplostionType
	
	waitTicks	as single
	
	musicHandle	as FSOUND_SAMPLE ptr
	musicChannel	as integer
	
	nSliders		as integer
	gradSlider(20)	as GradSliderType
	
	titleMID		as integer
	titleDL			as integer
	titleZoom		as single
	titleZoomSpeed	as single
	titleAngle		as single
	titleStartZoom	as single

end type


dim shared ExplosionGlobals as ExplosionGlobalsType


'-------------------------------------------------------------
' DeInit - include call inside Initialize.bas(DeInit)
'		   if you need to unload memory before program exits
'-------------------------------------------------------------
sub Explosion_DeInit ()
end sub



'-------------------------------------------------------------
' Initialize - load objects specific to this module in memory
'-------------------------------------------------------------
sub Explosion_Initialize ()

	ExplosionGlobals.initialized = 1
	
	dim as integer i
	
	ExplosionGlobals.nSliders = 20
	
	for i = 0 to ExplosionGlobals.nSliders - 1
		ExplosionGlobals.gradSlider(i).position.x = rnd * 10 - 5
		ExplosionGlobals.gradSlider(i).position.y = rnd * 10 - 5
		ExplosionGlobals.gradSlider(i).position.z = rnd * 10 - 5
		
		math_normalize3f ExplosionGlobals.gradSlider(i).position
		
		ExplosionGlobals.gradSlider(i).angle.x = rnd * 360
		ExplosionGlobals.gradSlider(i).angle.y = rnd * 360
		ExplosionGlobals.gradSlider(i).angle.z = rnd * 360
		
		ExplosionGlobals.gradSlider(i).velocity.x = rnd * 10 - 5
		ExplosionGlobals.gradSlider(i).velocity.y = rnd * 10 - 5
		ExplosionGlobals.gradSlider(i).velocity.z = rnd * 10 - 5
		
		math_normalize3f ExplosionGlobals.gradSlider(i).velocity
		
		ExplosionGlobals.gradSlider(i).velocityMag = rnd * 2 - 1
		
		
	next
	
	ExplosionGlobals.titleMID = objFile_load ("data/title.obj")
	
	ExplosionGlobals.titleZoom = 0
	ExplosionGlobals.titleZoomSpeed = 0
	ExplosionGlobals.titleAngle = 0
	
	ExplosionGlobals.titleStartZoom = system_time + .5

end sub



'-------------------------------------------------------------
' LaunchApp - used to call the module. on first run will call
'		      Initialize(). then setup module initial operating
'				  conditions
'-------------------------------------------------------------
sub Explosion_LaunchApp ()

	if ExplosionGlobals.initialized = 0 then Explosion_Initialize



	Explosion_MainLoop

	Explosion_ShutDown

end sub



'-------------------------------------------------------------
' MainLoop - module program loop
'-------------------------------------------------------------
sub Explosion_MainLoop ()


	do
		
		Explosion_ProcessInputs
		
		Explosion_Render
		
		glfwSwapBuffers
		
		system_upd_fpsr
		
	loop while ExplosionGlobals.exitCode = 0


end sub



'-------------------------------------------------------------
' ProcessInputs - Performs input processing.
'-------------------------------------------------------------
sub Explosion_ProcessInputs ()


	if glfwGetKey(GLFW_KEY_ESC) then ExplosionGlobals.exitCode = 1

	System_Process_Adjustment_Inputs

	if glfwGetKey(GLFW_KEY_SPACE) then ExplosionGlobals.titleZoom = 0

end sub



'-------------------------------------------------------------
' Render - root procedure for all module drawing during the
'          main loop
'-------------------------------------------------------------
sub Explosion_Render ()


	glClear  GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT

	glLoadIdentity
	glPushMatrix
	
	glDisable GL_CULL_FACE
	glDepthMask GL_FALSE
	glDisable GL_LIGHTING
	
	if ExplosionGlobals.titleStartZoom < system_time then

	glPushMatrix
		glTranslatef 0, 0, -50
		
		glRotatef ExplosionGlobals.gradSlider(0).angle.x, 1, 0, 0
		glRotatef ExplosionGlobals.gradSlider(0).angle.y, 0, 1, 0
		glRotatef ExplosionGlobals.gradSlider(0).angle.z, 0, 0, 1
		
		dim as integer i
		dim as single w0, posOfs
		
		w0 = 16
		posOfs = 10
		
		'----- draw scene
		
		for i = 0 to ExplosionGlobals.nSliders - 1
		
			glPushMatrix
				glTranslatef ExplosionGlobals.gradSlider(i).position.x * posOfs, ExplosionGlobals.gradSlider(i).position.y * posOfs, ExplosionGlobals.gradSlider(i).position.z * posOfs
				glRotatef ExplosionGlobals.gradSlider(i).angle.x, 1, 0, 0
				glRotatef ExplosionGlobals.gradSlider(i).angle.y, 0, 1, 0
				glRotatef ExplosionGlobals.gradSlider(i).angle.z, 0, 0, 1
				
				ExplosionGlobals.gradSlider(i).angle.x += ExplosionGlobals.gradSlider(i).velocityMag * 12 * ExplosionGlobals.gradSlider(i).velocity.x * system_ticksPassed / 1000
				ExplosionGlobals.gradSlider(i).angle.y += ExplosionGlobals.gradSlider(i).velocityMag * 12 * ExplosionGlobals.gradSlider(i).velocity.y * system_ticksPassed / 1000
				ExplosionGlobals.gradSlider(i).angle.z += ExplosionGlobals.gradSlider(i).velocityMag * 12 * ExplosionGlobals.gradSlider(i).velocity.z * system_ticksPassed / 1000
				
				glBegin GL_QUADS
				
					glColor4f 1, 1, 1, 0
					
					glVertex2f -w0, w0
					glVertex2f -w0, -w0
					
					glColor4f 1, 1, 1, ExplosionGlobals.titleZoom
					glVertex2f w0, -w0
					glVertex2f w0, w0
					
				glEnd
			glPopMatrix
			
		next
		
	glPopMatrix
	
	
		if ExplosionGlobals.titleZoom < 1 then ExplosionGlobals.titleZoom += (2 * system_ticksPassed / 1000) * (1 - ExplosionGlobals.titleZoom)
		
		if ExplosionGlobals.titleZoom > 1 then ExplosionGlobals.titleZoom = 1
		
		ExplosionGlobals.titleAngle += tempx * system_ticksPassed / 1000
		
		dim as integer nStp
		dim as single an0, xOfs, yOfs
		
		
		glPushMatrix
			glTranslatef 0, 1.28 * ExplosionGlobals.titleZoom, -18 * ExplosionGlobals.titleZoom
			
			glColor4f 0, 0, 0, ExplosionGlobals.titleZoom
			Render_Model_Plain ModelDB[ExplosionGlobals.titleMID].modelHandle
			
			glColor4f 0, 0, 0, .077 * ExplosionGlobals.titleZoom
			
			nStp = 20
			for i = 0 to nStp - 1
			
				an0 = i / nStp
				xOfs = .028 * cos(3.1415 * 2 * an0)
				yOfs = .028 * sin(3.1415 * 2 * an0)
				
				glPushmatrix
					glTranslatef xOfs, yOfs, 0
					Render_Model_Plain ModelDB[ExplosionGlobals.titleMID].modelHandle
				glPopMatrix
			
			next
			
		glPopMatrix
		
	end if
	
	
	
	
	glEnable GL_CULL_FACE
	glDepthMask GL_TRUE
	glEnable GL_LIGHTING
	
	system_2dplane_set_full
	
	
	system_2dplane_clear
	
	
	'----- end drawing
	
	
	glPopMatrix
	glFlush


end sub



'-------------------------------------------------------------
' Shutdown - called before exiting module to clear any initial
'            conditions that could affect other modules
'-------------------------------------------------------------
sub Explosion_Shutdown ()

	fsound_stopsound ExplosionGlobals.musicchannel
	fsound_close
end sub
