' Program:  Game Clock Routines
' Version:  0.04
' Language: FreeBASIC v0.18.3b
' Author:   Quinton Roberts (Eclipzer)
' Date:     11-13-06
' Update:   03-25-08
'
' Copyright (c) 2006-2008.

' clock constants
  const IO_FORWARD=1
  const IO_BACKWARD=0
  
' clock flags  
  const fUPDATE=1 '0=off 1=update
  const fCOUNT=2  '0=backward 1=forward
  const fALARM=3  '0=off 1=triggered  

' Data Structures  
  type clockdat    
    as integer flags  'clock flags (update,count,alarm)
    as double  real   'timer sample    
    as double  epoch  'initial time sample (base time)
    as double  lapse  'sampled difference between timer and epoch
    as double  start  'static start time value (user defined)
    as double  alarm  '       alarm time value (user defined)
    as double  value  'dynamic start time value
    as double  curr   'current time    
  end type
  
' Declarations  
  declare function clock_state(t as clockdat) as string
  
  declare function io_bit    (byref vdat as uinteger,idx as uinteger) as uinteger
  declare sub      io_bit_clr(byref vdat as uinteger,idx as uinteger)
  declare sub      io_bit_flp(byref vdat as uinteger,idx as uinteger)
  declare function io_bit_get(byref vdat as uinteger,idx as uinteger,bits as uinteger) as uinteger
  declare sub      io_bit_put(byref vdat as uinteger,idx as uinteger,bits as uinteger,value as uinteger)  
  declare sub      io_bit_set(byref vdat as uinteger,idx as uinteger)
  
  declare sub io_clock_inc    (t as clockdat,hh as double=0,mm as double=0,ss as double=0)
  declare sub io_clock_pause  (t as clockdat)
  declare sub io_clock_set    (t as clockdat,hh as double=0,mm as double=0,ss as double=0)
  declare sub io_clock_start  (t as clockdat,update as integer=1)
  declare sub io_clock_stop   (t as clockdat)
  declare sub io_clock_update (t as clockdat)
  
  declare function io_time_hms(tt as double) as string
  
  #ifdef 0
  const CLOCKS=4
    
  dim shared clock(CLOCKS-1) as clockdat
    
  dim as string  kb,state,div1,div2,div3
  dim as integer i,done,xx,yy,idx=0
  
  Screen    FX_640x480,32,3,0
  ScreenSet FX_WRK_PAGE,FX_VIS_PAGE
  
  for i=0 to CLOCKS-1
    io_clock_set(clock(i))    
    io_clock_start(clock(i)) 'init game clock at 0  
  next  
  
  div1=" -----------------------------[ ||| eclipzer ||| ]-----------------------------"
  div2=" +-----------------------------------------------------------------+"
  div3=" |                                                                 |"
    
    
  clock(0).alarm=0*3600+0*60+3.25 'set time value
  clock(1).alarm=0*3600+0*60+10.07 'set time value
  clock(2).alarm=0*3600+0*60+25 'set time value
  
  
  io_clock_set clock(3),,,30
  io_clock_start clock(3),IO_BACKWARD
  clock(3).alarm=0 'set time value
  
  do
    
    for i=0 to CLOCKS-1
      io_clock_update(clock(i)) 'update game clock each cycle   
    next
    
    kb=inkey$
    
    select case kb
    case chr$(27):  done=1
    case "+":       io_clock_inc(clock(idx),,,15)
    case "-":       io_clock_inc(clock(idx),,,-15)    
    case " ":       io_clock_pause(clock(idx))
    case "x","X":   io_clock_stop(clock(idx))
    case chr$(13):  io_clock_set(clock(idx),,,clock(idx).start)
    case ASC_LEFT:  io_clock_pause(clock(idx)): io_clock_start(clock(idx),IO_BACKWARD)
    case ASC_RIGHT: io_clock_pause(clock(idx)): io_clock_start(clock(idx),IO_FORWARD)
    case ASC_UP:    idx=(idx-1) and (CLOCKS-1)
    case ASC_DOWN:  idx=(idx+1) and (CLOCKS-1)
    case else
    end select
    
    ScreenCopy FX_CLR_PAGE,FX_WRK_PAGE
    
    locate 1,1
    ?
    ? " Game Clock Routines"
    ? div1
    ?
    ? div2 ' +----+
    for i=0 to CLOCKS      
      ? div3 ' |    |
    next
    ? div2 ' +----+
    ?
    ? " Key Controls"
    ? "  UP ....... Cycle active clock (backward)"
    ? "  DOWN ..... Cycle active clock (forward)"
    ? "  LEFT ..... Set active clock to count backward"
    ? "  RIGHT .... Set active clock to count forward"    
    ? "  SPACE .... Toggle active clock on/off (pause)"
    ? "  ENTER .... Reset active clock"
    ? "  X ........ Stop active clock"
    ? "  + ........ Add 15 seconds to active clock"
    ? "  - ........ Subtract 15 seconds from active clock"
    ? "  ESC ...... Exit"
    ?
    
    xx=4
    yy=6
    locate yy,xx: ? "Clock      Time         Start         Alarm      State"
    
    for i=0 to CLOCKS-1    
      locate   ,xx: ? "  "& i &"    "+io_time_hms(clock(i).curr)+"  "+io_time_hms(clock(i).start)+"  "+io_time_hms(clock(i).alarm)+"  "+clock_state(clock(i))
      if io_bit(clock(i).flags,fALARM) then io_clock_stop(clock(i))
    next
    
    locate yy+idx+1,xx: ? ">" 'active clock cursor    
    
    locate 27,1
    ? div1
    ?
    ? " Copyright (c) Quinton Roberts 2006-2008."

    ScreenCopy
    
  loop until done
  
  #endif
  
  function clock_state(t as clockdat) as string
    
    select case io_bit(t.flags,fUPDATE)'t.update
    case 0: return "Paused"
    case 1
      select case io_bit(t.flags,fCOUNT)
      case IO_FORWARD:  return "Count Forward"
      case IO_BACKWARD: return "Count Backward"
      case else:        return "Paused"
      end select
    end select
    
  end function  
  
' =============================================================================
'          Name: io_bit (02.21.08)
'       Returns: 
'    Parameters:
'          vdat: variable that contains bit data
'           idx: bit position
' -----------------------------------------------------------------------------
'   Description: return selected bit value (0=clear 1=set).
'      Comments: 
' =============================================================================  
  function io_bit (byref vdat as uinteger,idx as uinteger) as uinteger
    return (vdat shr idx) and 1
  end function
  
' =============================================================================
'          Name: io_bit_clr (02.21.08)
'       Returns: 
'    Parameters:
'          vdat: variable that contains bit data
'           idx: bit position
' -----------------------------------------------------------------------------
'   Description: Clear selected bit (value=0).
'      Comments: 
' =============================================================================  
  sub io_bit_clr(byref vdat as uinteger,idx as uinteger)
    vdat=(vdat and (not (1 SHL idx))) 'clear selected bit (set to 0)
  end sub
  
' =============================================================================
'          Name: io_bit_flp (02.21.08)
'       Returns: 
'    Parameters:
'          vdat: variable that contains bit data
'           idx: bit position
' -----------------------------------------------------------------------------
'   Description: Toggle selected bit.
'      Comments: Toggles a bit between clear and set values. A clear bit
'                becomes set (1), while a set bit becomes clear (0).
' =============================================================================  
  sub io_bit_flp(byref vdat as uinteger,idx as uinteger)
    vdat=(vdat xor (1 SHL idx)) 'toggle selected bit
  end sub
  
' =============================================================================
'          Name: io_bit_get (02.21.08)
'       Returns: 
'    Parameters:
'          vdat: variable that contains bit data
'           idx: bit position
'          bits: number of bits to return (sequence length)
' -----------------------------------------------------------------------------
'   Description: Return a sequence of bits from a variable.
'      Comments: Useful for extracting packed bit information.
' =============================================================================  
  function io_bit_get(byref vdat as uinteger,idx as uinteger,bits as uinteger) as uinteger
    return (vdat SHR idx) and ((2^bits)-1) 'retrieve bits at selected bit
  end function
  
' =============================================================================
'          Name: io_bit_put (02.21.08)
'       Returns: 
'    Parameters:
'          vdat: variable that contains bits data
'           idx: bit position
'          bits: number of bits to replace
'         value: replacement value
' -----------------------------------------------------------------------------
'   Description: Replace a sequence of bits in a variable.
'      Comments: Useful for inserting packed bit information.
' =============================================================================  
  sub io_bit_put(byref vdat as uinteger,idx as uinteger,bits as uinteger,value as uinteger)
    vdat=(vdat and (not((2^bits)-1) SHL idx)) 'clear selected bits
    vdat=(vdat or  (value SHL idx))           'store value at selected bit
  end sub
  
' =============================================================================
'          Name: io_bit_set (02.21.08)
'       Returns: 
'    Parameters:
'          vdat: variable that contains bit data
'           idx: bit position
' -----------------------------------------------------------------------------
'   Description: Set selected bit (value=1).
'      Comments: 
' =============================================================================  
  sub io_bit_set(byref vdat as uinteger,idx as uinteger)
    vdat=(vdat or (1 SHL idx)) 'set selected bit (set to 1)
  end sub

' =============================================================================
'          Name: io_clock_inc (03.25.08)
'       Returns: 
'    Parameters:
'             t: time structure
'            hh: init hours [0]
'            mm: init minutes [0]
'            ss: init seconds [0]
' -----------------------------------------------------------------------------
'   Description: Add set amount of time to play clock.
'      Comments: 
' =============================================================================
  sub io_clock_inc(t as clockdat,hh as double,mm as double,ss as double)
    t.value+=hh*3600+mm*60+ss    
  end sub
  
' =============================================================================
'          Name: io_clock_pause (03.25.08)
'       Returns: 
'    Parameters:
'             t: time structure
' -----------------------------------------------------------------------------
'   Description: Toggle time structure for updating.
'      Comments: 
' =============================================================================
  sub io_clock_pause(t as clockdat)
    io_bit_flp(t.flags,fUPDATE)    
    select case io_bit(t.flags,fUPDATE)
    case 0: t.value=t.curr: t.lapse=0
    case 1: io_clock_start(t,io_bit(t.flags,fCOUNT))
    end select
  end sub

' =============================================================================
'          Name: io_clock_set (03.25.08)
'       Returns: 
'    Parameters:
'             t: time structure
'            hh: init hours [0]
'            mm: init minutes [0]
'            ss: init seconds [0]
' -----------------------------------------------------------------------------
'   Description: Set a specific time for a time structure.
'      Comments: Only the time is set. To actually start the clock, use the
'                io_clock_start routine.
' =============================================================================
  sub io_clock_set(t as clockdat,hh as double,mm as double,ss as double)
    t.epoch=timer            'set new base time
    t.start=hh*3600+mm*60+ss 'set start time
    t.value=t.start          'set time value
  end sub

' =============================================================================
'          Name: io_clock_start (03.25.08)
'       Returns: 
'    Parameters:
'             t: time structure
'         count: counting direction 0=backward 1=forward [1]
' -----------------------------------------------------------------------------
'   Description: Init time structure for updating.
'      Comments: Call this routine to start a clock structure. 
' =============================================================================  
  sub io_clock_start(t as clockdat,count as integer=1)
    t.epoch=timer                      'set new base time
    io_bit_put(t.flags,fCOUNT,1,count) 'set clock direction    
    io_bit_set(t.flags,fUPDATE)        'set clock update flag   
  end sub

' =============================================================================
'          Name: io_clock_stop (03.25.08)
'       Returns: 
'    Parameters:
'             t: time structure
' -----------------------------------------------------------------------------
'   Description: Prevent time structure from updating.
'      Comments: 
' =============================================================================       
  sub io_clock_stop(t as clockdat)     
    t.value=t.curr              'save current time
    t.lapse=0                   'reset time lapse
    io_bit_clr(t.flags,fUPDATE) 'clear update flag   
  end sub  

' =============================================================================
'          Name: io_clock_update (03.25.08)
'       Returns: 
'    Parameters:
'             t: time structure
' -----------------------------------------------------------------------------
'   Description: Update current time structure.
'      Comments: This should be called once every program cycle. 
' =============================================================================
  sub io_clock_update(t as clockdat)
    
    dim as integer sign,alarm_=0
    
    t.real=timer
    
    if io_bit(t.flags,fUPDATE) then 
      select case io_bit(t.flags,fCOUNT)
      case IO_FORWARD:  sign=1
      case IO_BACKWARD: sign=-1
      end select    
      t.lapse=sign*(t.real-t.epoch)
    end if
    
    t.curr=t.value+t.lapse     'update current time
    
    io_bit_clr(t.flags,fALARM) 'clear alarm flag
    
    select case t.start
    case is>t.alarm: if t.curr<=t.alarm then alarm_=1
    case is<t.alarm: if t.curr>=t.alarm then alarm_=1
    case else
    end select
    
    if alarm_ then io_bit_set(t.flags,fALARM): io_clock_set(t,,,t.alarm)
    
  end sub

' =============================================================================
'          Name: io_time_hms (03.23.08)
'       Returns: time string
'    Parameters:
'            tt: time, in seconds, to convert
' -----------------------------------------------------------------------------
'   Description: Converts seconds to a string in HH:MM:SS.SSS format.
'      Comments: 
' =============================================================================     
  function io_time_hms(tt as double) as string            
    
    dim as integer hh=fix(tt/3600): tt-=3600*hh 
    dim as integer mm=fix(tt/60):   tt-=60*mm
    dim as integer ss=fix(tt)
    dim as integer ms=int(abs(frac(tt)*1000))
      
    dim as string  hhn=str$(hh): if len(hhn)<2 then hhn="0"+hhn
    dim as string  mmn=str$(mm): if len(mmn)<2 then mmn="0"+mmn
    dim as string  ssn=str$(ss): if len(ssn)<2 then ssn="0"+ssn
    dim as string  msn=str$(ms): if len(msn)<2 then msn="0"+msn
                                if len(msn)<3 then msn="0"+msn                  
    
    return hhn+":"+mmn+":"+ssn+"."+msn
    
  end function 
  
  
