type iconType

	MID		as integer
	DL		as integer
	
end type


type GPTriangleType
	point_id(1 to 3) as integer
	normal(1 to 3)	as integer
	vnormal(1 to 3) as vector3d
	color(1 to 3)	as rgbalpha
	uv(1 to 3)		as UV2D
end type


type GPMeshType
	max_vertices	as integer
	max_triangles	as integer
	vertices		as Vector3D ptr
	triangles		as GPTriangleType ptr
	meanVertex		as Vector3D
	stdDeviation	as single
end type


type GenProjectileType
	active			as integer
	class			as integer
	matrix(16)		as single
	position		as Vector3D
	lock			as integer
	angle			as Vector3D
	angularVelocity	as Vector3D
	velocity		as Vector3D
	accelVector		as Vector3D
	accelMag		as single
	mesh			as GPMeshType
	blockSize		as single
end type


type GeneratorColorType
	position		as Vector3D
	fallOff			as single
	color			as rgbAlpha
end type
	

type GeneratorClassType
	title			as string
	class			as integer
	position		as Vector3D
	angle			as Vector3D
	mass			as single
	preAngularVelocity			as Vector3D
	preAngularVelocityAngle		as Vector3D
	
	'--- block torus specific
	blockSize		as single
	length			as single
	innerRadius		as single
	outterRadius	as single
	
	'--- hex tunnel specific
	radius			as single
	iterations		as single
	rows			as single
	cellSpacing		as single
	cellBorder		as integer
	textureScrollSpeed	as single
	rowsReturn		as single
	textureScroll	as single
	
	'--- meta ball cloud specific
	cloudSize		as single
	
	nProjectiles		as integer
	nProjectilesReturn	as single
	projectile			as GenProjectileType ptr
	
	nVicinityColors as integer
	vicinityColor	as GeneratorColorType ptr
	
	nColorLights	as integer
	colorLight		as GeneratorColorType ptr
	
	shadowColor		as rgbAlpha
end type



type WaveSpawnType
	active			as integer
	position		as Vector3D
	angle			as Vector3D
	health			as single
	nWayPoints		as integer
	nextWayPoint	as integer
	velocity		as Vector3D
	targetVelocity	as Vector3D
	spawnTime		as single
	lastFire		as single
	detonate		as integer
	orbitalClass	as integer
	nGenerators		as integer
	generator		as GeneratorClassType ptr
	totalProjectiles	as integer
	spawnIndex		as integer
	ringAngle		as Vector3D
	shield_on		as integer
	pulse			as single			'color pulse when hit. 1 = 100% alt color
end type


type WaveClassType
	title			as string
	time			as single
	speed			as single
	class			as integer
	pattern			as integer
	nSpawn			as integer
	nSpawnReturn	as single
	spawn			as WaveSpawnType ptr
	position		as Vector3D
	positionOffset	as Vector3D
	startTime		as single
	maxHealth		as single
	rateOfFire		as single
	bulletPattern	as integer
	bulletClass		as integer
	
	'--- dragon specific
	radius			as single
	orbital2_figure8_swap			as integer
	orbital2_figure8_swap_next		as integer
	shield_on		as integer
	shield_ticks	as integer
	shield_wait_len	as integer
	nActive			as integer
	nextBomb		as single

end type



type PhaseSequenceType
	roomClass		as integer  'RC_SPHERE, RC_TUNNEL
	phaseTitle		as string
	nGenerators		as integer
	generator		as GeneratorClassType ptr
	nWaves			as integer
	wave			as WaveClassType ptr
	
	startTime		as single
	startTimeOffset	as single
	
	bossActive		as integer
	bossWave		as integer
	
	'--- sphere specific camera
	cameraOffset	as single	'outter sphere cam distance from center
	cameraOffsetH	as single	'horizontal angle
	cameraOffsetV	as single	'vertical angle
	cameraVelocityH	as single	'h velocity
	cameraVelocityV	as single	'v velocity
	'----
	
	totalHealth		as integer
	totalSpawn		as integer
	totalProjectiles	as integer
	totalPowerUps	as integer
	
end type


type PhaseDataGlobalsType
	phaseSequence	as PhaseSequenceType
	
end type

dim shared PhaseDataGlobals as PhaseDataGlobalsType



declare sub PhaseData_Calculate_Totals ()

declare sub PhaseData_Generator_Load (gen as GeneratorClassType ptr, filenum as integer)

declare sub PhaseData_Init_Block_Torus_Regenerate (gen as generatorClassType ptr)
declare sub PhaseData_Init_Hex_Tunnel_Regenerate (gen as generatorClassType ptr)
declare sub PhaseData_Init_Meta_Ball_Cloud_Regenerate (gen as generatorClassType ptr)
declare sub PhaseData_Init_Meta_Ball_Disc_Regenerate (gen as generatorClassType ptr)

declare sub PhaseData_Init_Wave_Regenerate (wave as WaveClassType ptr)
declare sub PhaseData_Init_Wave_Regenerate_Spawn_BossDragon (wave as WaveClassType ptr, spawn as WaveSpawnType ptr)
declare sub PhaseData_Init_Wave_Regenerate_Spawn_Flies (wave as WaveClassType ptr, spawn as WaveSpawnType ptr)

declare sub PhaseData_PhaseSequence_Load (filename as string)
declare sub PhaseData_PhaseSequence_Unload ()


declare sub PhaseData_Projectile_Set_Mesh (gen as GeneratorClassType ptr, proj as GenProjectileType ptr, gpClass as integer)
declare sub PhaseData_Projectile_Set_Mesh_Vertex (byref mesh as GPMeshType ptr, index as integer, x as single, y as single, z as single)
declare sub PhaseData_Projectile_Set_Mesh_Triangle_Point_ID (byref mesh as GPMeshType ptr, index as integer, a as integer, b as integer, c as integer)
declare sub PhaseData_Projectile_Set_Mesh_Triangle_Point_Normals (byref mesh as GPMeshType ptr, index as integer)
declare sub PhaseData_Projectile_Calc_Colors (gen as GeneratorClassType ptr, proj as GenProjectileType ptr)
declare sub PhaseData_Projectile_Calc_Colors_Vicinity (byval light0 as Vector3D, byval light0c as rgbalpha, byval v1 as vector3d, byref c1 as rgbalpha, fallOff as single)
declare sub PhaseData_Projectile_Calc_Colors_Angle (byval light0 as Vector3D, byval light0c as rgbalpha, byref light1c as rgbalpha, byval v1 as vector3d, byval c1 as rgbalpha, byval n1 as vector3d, fallOff as single)


declare sub PhaseData_Projectile_Calc_MeanVertex (proj as GenProjectileType ptr)
declare sub PhaseData_Projectile_Calc_stdDeviation (proj as GenProjectileType ptr)
declare sub PhaseData_Projectile_Calc_totalDeviation (proj as GenProjectileType ptr)

declare sub PhaseData_Wave_Load (wave as WaveClassType ptr, filenum as integer)


#DEFINE GC_CAMERA				0
#DEFINE GC_BLOCK_TORUS			1
#DEFINE GC_HEX_TUNNEL			2
#DEFINE GC_META_BALL_CLOUD		3
#DEFINE GC_META_BALL_DISC		4


	
'---- generator projectile classes
#DEFINE GPC_BLOCK_TORUS_BLOCK	1
#DEFINE GPC_META_BALL			2
#DEFINE GPC_RING_WEDGE			3
#DEFINE GPC_RING_SPIKE			4
#DEFINE GPC_EXPLOSION_META_BALL	5
#DEFINE GPC_BALL_BLOCK			6
#DEFINE GPC_FIN_LEFT			7
#DEFINE GPC_FIN_RIGHT			8
#DEFINE GPC_HEX_TUNNEL_HEX		9


#DEFINE RC_COLOR_LIGHT			0
#DEFINE RC_VICINITY_COLOR		1


'---- wave classes
#DEFINE WC_FLIES				0
#DEFINE WC_BOSS_DRAGON			1


'---- wave patterns
#DEFINE WP_TYPE_I				0
#DEFINE WP_ORBITAL				1
#DEFINE WP_ORBITAL_TYPE_II		2

'---- room classes
#DEFINE RC_SPHERE				0
#DEFINE RC_TUNNEL				1
