DECLARE sub PhaseEditor_DeInit ()
DECLARE sub PhaseEditor_Initialize ()
DECLARE sub PhaseEditor_LaunchApp ()
DECLARE sub PhaseEditor_MainLoop ()
DECLARE sub PhaseEditor_ProcessInputs ()
DECLARE sub PhaseEditor_Render ()
DECLARE sub PhaseEditor_Shutdown ()


'--------------------
'  Globals - globals for this module are stored in this type then shared
'--------------------

type PhaseEditorGlobalsType
	initialized		as byte
	exitCode		as byte
	
	mousex			as integer
	mousey			as integer
	mouseLeft		as integer
	mouseRight		as integer
	
	curWindow		as integer

	winAlpha(99)	as single
	winActive(99)	as integer
	winLock(99)		as integer
	
	generatorClass(99)		as GeneratorClassType
	generatorClassLast		as integer
	generatorClassSelected	as integer
	
	waveClass(99)		as WaveClassType
	waveClassLast		as integer
	waveClassSelected	as integer
	
	wavePattern(99)		as WaveClassType
	wavePatternLast		as integer
	wavePatternSelected	as integer
	
	bulletClass(99)		as bulletClassType
	bulletClassLast		as integer
	bulletClassSelected	as integer
	
	bulletPattern(99)		as bulletClassType
	bulletPatternLast		as integer
	bulletPatternSelected	as integer
	
	winCoorBox			as WinCoorBoxType
	winNumberBox		as WinNumberBoxType
	winSelectColor		as WinSelectColorType
	winSelectTable		as WinSelectTableType
	winSequencer		as WinSequencerType
	winTextBox			as WinTextBoxType
	
	winNewMesh			as WinNewMeshType
	winNewPhase			as WinNewPhaseType
	winNewWave			as WinNewWaveType
	
	winMeshProperties	as WinMeshPropertiesType
	winWaveProperties	as WinWavePropertiesType

	windowLock		as integer
	waitTicks		as integer
	
	chromeTex		as integer
	
end type


dim shared PhaseEditorGlobals as PhaseEditorGlobalsType


'-------------------------------------------------------------
' DeInit - include call inside Initialize.bas(DeInit)
'		   if you need to unload memory before program exits
'-------------------------------------------------------------
sub PhaseEditor_DeInit ()

	
	
end sub



'-------------------------------------------------------------
' Initialize - load objects specific to this module in memory
'-------------------------------------------------------------
sub PhaseEditor_Initialize ()

	PhaseEditorGlobals.initialized = 1
	
	PhaseDataGlobals.PhaseSequence.nWaves = 0
	PhaseDataGlobals.PhaseSequence.wave = callocate(sizeof(WaveClassType))
	PhaseDataGlobals.phaseSequence.nGenerators = 1
	PhaseDataGlobals.phaseSequence.generator = callocate(sizeof(GeneratorClassType))
	
	PhaseEditorGlobals.winActive(WIN_SEQUENCER) = 1
	PhaseEditorGlobals.winLock(WIN_SEQUENCER) = 1
	PhaseEditorGlobals.winActive(WIN_NEW_PHASE) = 1
	
	PhaseEditor_Window_Init_NewPhase
	
	
	system_ticks = system_time * 1000
	
	PhaseEditor_Init_WaveClasses
	PhaseEditor_Init_WavePatterns
	PhaseEditor_Init_BulletClasses
	PhaseEditor_Init_BulletPatterns
	PhaseEditor_Init_GeneratorClasses
	PhaseEditor_Init_Camera
	
	
	Load_TextureL blankTex, "data\blankTex.png"
	Load_Texture  blendTex, "data\blankTex.png"
	
	Load_Texture  PhaseEditorGlobals.chromeTex, "data\chrome.png"
	
	glfwSetWindowTitle "Prompt Critical - Phase Sequence Editor"
	
end sub



'-------------------------------------------------------------
' LaunchApp - used to call the module. on first run will call
'		      Initialize(). then setup module initial operating
'				  conditions
'-------------------------------------------------------------
sub PhaseEditor_LaunchApp ()

	if PhaseEditorGlobals.initialized = 0 then PhaseEditor_Initialize



	PhaseEditor_MainLoop

	PhaseEditor_ShutDown

end sub



'-------------------------------------------------------------
' MainLoop - module program loop
'-------------------------------------------------------------
sub PhaseEditor_MainLoop ()


	do
		
		PhaseEditor_ProcessInputs
		
		PhaseEditor_Render
		
		glfwSwapBuffers
		
		system_upd_fpsr
		
	loop while PhaseEditorGlobals.exitCode = 0


end sub



'-------------------------------------------------------------
' ProcessInputs - Performs input processing.
'-------------------------------------------------------------
sub PhaseEditor_ProcessInputs ()


	System_Process_Adjustment_Inputs
	
	
	'--- check mouse input
	glfwGetMousePos @PhaseEditorGlobals.mousex, @PhaseEditorGlobals.mousey
	
	
	'--- handle key input
	System_Process_Key_Strokes

end sub



'-------------------------------------------------------------
' Render - root procedure for all module drawing during the
'          main loop
'-------------------------------------------------------------
sub PhaseEditor_Render ()


	glClear  GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT

	glLoadIdentity
	glPushMatrix
	
		PhaseEditor_Render_Scene
		PhaseEditor_Render_Gui_Layer
	
	
	glPopMatrix
	glFlush


end sub



'-------------------------------------------------------------
' Shutdown - called before exiting module to clear any initial
'            conditions that could affect other modules
'-------------------------------------------------------------
sub PhaseEditor_Shutdown ()

	PhaseData_PhaseSequence_Unload
	
end sub
