DECLARE sub PhaseMenu_DeInit ()
DECLARE sub PhaseMenu_Initialize ()
DECLARE sub PhaseMenu_LaunchApp ()
DECLARE sub PhaseMenu_MainLoop ()
DECLARE sub PhaseMenu_ProcessInputs ()
DECLARE sub PhaseMenu_Render ()
DECLARE sub PhaseMenu_Shutdown ()


'--------------------
'  Globals - globals for this module are stored in this type then shared
'--------------------

'-------------------------------------------------------------
' DeInit - include call inside Initialize.bas(DeInit)
'		   if you need to unload memory before program exits
'-------------------------------------------------------------
sub PhaseMenu_DeInit ()
end sub



'-------------------------------------------------------------
' Initialize - load objects specific to this module in memory
'-------------------------------------------------------------
sub PhaseMenu_Initialize ()

	PhaseMenuGlobals.initialized = 1
	PhaseMenuGlobals.exitCode = 0
	
	dim as integer i
	
	PhaseMenuGlobals.nSliders = 20
	
	for i = 0 to PhaseMenuGlobals.nSliders - 1
		PhaseMenuGlobals.gradSlider(i).position.x = rnd * 10 - 5
		PhaseMenuGlobals.gradSlider(i).position.y = rnd * 10 - 5
		PhaseMenuGlobals.gradSlider(i).position.z = rnd * 10 - 5
		
		math_normalize3f PhaseMenuGlobals.gradSlider(i).position
		
		PhaseMenuGlobals.gradSlider(i).angle.x = rnd * 360
		PhaseMenuGlobals.gradSlider(i).angle.y = rnd * 360
		PhaseMenuGlobals.gradSlider(i).angle.z = rnd * 360
		
		PhaseMenuGlobals.gradSlider(i).velocity.x = rnd * 10 - 5
		PhaseMenuGlobals.gradSlider(i).velocity.y = rnd * 10 - 5
		PhaseMenuGlobals.gradSlider(i).velocity.z = rnd * 10 - 5
		
		math_normalize3f PhaseMenuGlobals.gradSlider(i).velocity
		
		PhaseMenuGlobals.gradSlider(i).velocityMag = rnd * 2 - 1
		
		
	next
	
	PhaseMenuGlobals.titleMID = objFile_load ("data/title.obj")
	PhaseMenuGlobals.waveMID = objFile_load ("data/wave.obj")
	PhaseMenuGlobals.gameOverMID = objFile_load ("data/gameover.obj")
	PhaseMenuGlobals.GOOOHMID = objFile_load ("data/gameover_ooh.obj")
	PhaseMenuGlobals.GOOOTMID = objFile_load ("data/gameover_oot.obj")
	PhaseMenuGlobals.scoreMID = objFile_load ("data/score.obj")
	PhaseMenuGlobals.theEndMID = objFile_load ("data/theend.obj")
	
	PhaseMenuGlobals.hud_healthMID = objFile_load ("data/hud_health.obj")
	PhaseMenuGlobals.hud_clockMID = objFile_load ("data/hud_clock.obj")
	PhaseMenuGlobals.hud_ammoMID = objFile_load ("data/hud_ammo.obj")
	PhaseMenuGlobals.hud_shieldMID = objFile_load ("data/hud_shield.obj")
	
	PhasePlayer_HighScores_Load
	
	PhasePlayer_Init_Music
	glfwSetWindowTitle "Prompt Critical"
	
	PhasePlayer_Init_Missiles
	PhasePlayer_Init_Weapons
	PhasePlayer_Init_Explosions
	PhasePlayer_Init_Bullets
	PhasePlayer_Init_7Segment
	
	Load_TextureL blankTex, "data\blankTex.png"
	Load_Texture  blendTex, "data\blankTex.png"
	
	Load_TextureM PhasePlayerGlobals.chromeTex, "data\chrome.png"
	
	glDisable GL_TEXTURE_2D
	
	
	PhaseEditorGlobals.winActive(WIN_MAIN) = 1
	PhaseEditorGlobals.winLock(WIN_MAIN) = 0
	
	
	PhaseMenuGlobals.titleZoom = 0
	PhaseMenuGlobals.titleZoomSpeed = 0
	PhaseMenuGlobals.titleAngle = 0
	PhaseMenuGlobals.titleStartZoom = system_time + .5
	PhaseMenuGlobals.waitTicks = system_time * 1000 + 500
	
	PhaseMenuGlobals.nVideoModes = glfwGetVideoModes (@PhaseMenuGlobals.videoMode(0), 100)
	
	PhaseMenuGlobals.winOptions.res.x = OpenGLProperties.SCR_WIDTH
	PhaseMenuGlobals.winOptions.res.y = OpenGLProperties.SCR_HEIGHT
	if OpenGLProperties.SCR_BITS = 16 then PhaseMenuGlobals.winOptions.bitDepth = 0
	if OpenGLProperties.SCR_BITS = 32 then PhaseMenuGlobals.winOptions.bitDepth = 1
	
	PhaseMenuGlobals.winOptions.fullScreen = OpenGLProperties.SCR_FULLSCREEN
	
	if OpenGLProperties.SCR_FULLSCREEN = GLFW_WINDOW then PhaseMenuGlobals.winOptions.fullScreen = 0
	if OpenGLProperties.SCR_FULLSCREEN = GLFW_FULLSCREEN then PhaseMenuGlobals.winOptions.fullScreen = 1
	
	dim as integer a, nBits
	a = 0
	for i = 0 to PhaseMenuGlobals.nVideoModes - 1
		if PhaseMenuGlobals.videoMode(i).height >= 400 then
		nBits = PhaseMenuGlobals.videoMode(i).redBits + PhaseMenuGlobals.videoMode(i).greenBits + PhaseMenuGlobals.videoMode(i).blueBits
		if  nBits = 24 and PhaseMenuGlobals.winOptions.bitDepth = 1 then a += 1
		if  nBits = 16 and PhaseMenuGlobals.winOptions.bitDepth = 0 then a += 1
		if PhaseMenuGlobals.winOptions.res.x = PhaseMenuGlobals.videoMode(i).width and PhaseMenuGlobals.winOptions.res.y = PhaseMenuGlobals.videoMode(i).height then
			if  nBits = 24 and PhaseMenuGlobals.winOptions.bitDepth = 1 then PhaseMenuGlobals.winOptions.resolution = a
			if  nBits = 16 and PhaseMenuGlobals.winOptions.bitDepth = 0 then PhaseMenuGlobals.winOptions.resolution = a
		end if
		end if
	next
	
	if PhaseMenuGlobals.winOptions.sfxOn = 1 then 
		PhasePlayerGlobals.musicChannel(MH_SUB_DROP) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_SUB_DROP))
		FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_SUB_DROP), PhaseMenuGlobals.winOptions.sfxVol
	end if
	
end sub



'-------------------------------------------------------------
' LaunchApp - used to call the module. on first run will call
'		      Initialize(). then setup module initial operating
'				  conditions
'-------------------------------------------------------------
sub PhaseMenu_LaunchApp ()

	if PhaseMenuGlobals.initialized = 0 then PhaseMenu_Initialize


	
	PhaseMenu_MainLoop

	PhaseMenu_ShutDown

end sub



'-------------------------------------------------------------
' MainLoop - module program loop
'-------------------------------------------------------------
sub PhaseMenu_MainLoop ()


	do
		PhaseMenu_ProcessInputs
		
		PhaseMenu_Render
		if PhaseMenuGlobals.titleZoom > .8 then PhaseMenu_Render_Gui_Layer
		
		glfwSwapBuffers
		
		system_upd_fpsr
		
		if PhaseMenuGlobals.restart = 1 then
			glfwTerminate
			Initialize_GameConfig
			Initialize_GLFW
			Initialize_OpenGL
			
			system_virtualTime = 0
			system_upd_fpsr
			PhaseMenuGlobals.restart = 0
			PhaseeditorGlobals.winAlpha(WIN_OPTIONS) = 0
			
			PhaseMenuGlobals.titleZoom = 0
			PhaseMenuGlobals.titleZoomSpeed = 0
			PhaseMenuGlobals.titleAngle = 0
			PhaseMenuGlobals.titleStartZoom = system_time + .5
			PhaseMenuGlobals.waitTicks = system_time * 1000 + 200
			PhaseEditorGlobals.waitTicks = system_time * 1000 + 200
			PhasePlayerGlobals.waitTicks = system_time * 1000 + 200
			
			PhasePlayerGlobals.player.fireSoundEnd = 0
			PhasePlayerGlobals.player.ricochetEnd = 0
			PhasePlayerGlobals.player.alarmEnd = 0
			PhasePlayerGlobals.player.fuzzTime = 0
			
			system_ticks = system_time * 1000
			system_ticksPassed = 0
			fpsr = 0
			
			Load_TextureL blankTex, "data\blankTex.png"
			Load_Texture  blendTex, "data\blankTex.png"
			
			Load_TextureM PhasePlayerGlobals.chromeTex, "data\chrome.png"
			
			glDisable GL_TEXTURE_2D
			
			
			if PhaseMenuGlobals.winOptions.sfxOn = 1 then 
				PhasePlayerGlobals.musicChannel(MH_SUB_DROP) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_SUB_DROP))
				FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_SUB_DROP), PhaseMenuGlobals.winOptions.sfxVol
			end if
		end if
		
	loop while PhaseMenuGlobals.exitCode = 0


end sub



'-------------------------------------------------------------
' ProcessInputs - Performs input processing.
'-------------------------------------------------------------
sub PhaseMenu_ProcessInputs ()


	dim as integer i
	
	System_Process_Adjustment_Inputs
	
	
	'--- check mouse input
	glfwGetMousePos @PhaseEditorGlobals.mousex, @PhaseEditorGlobals.mousey
	
	
	'--- handle key input
	System_Process_Key_Strokes
	
	if PhaseMenuGlobals.winOptions.musicOn = 1 then 
		if PhasePlayerGlobals.player.musicPlaying = 0 then 
			PhasePlayerGlobals.musicChannel(MH_SONG2) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_SONG2))
			PhasePlayerGlobals.player.musicPlaying = 1
		end if
		FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_SONG2), PhaseMenuGlobals.winOptions.musicVol * PhaseMenuGlobals.titleZoom
	end if
	
	if PhaseMenuGlobals.waitTicks > system_ticks then exit sub
	
	if PhaseMenuGlobals.destination = D_GAME and PhaseMenuGlobals.titleFade = 1 and PhaseMenuGlobals.titleZoom = 0 then
	
		fsound_stopsound PhasePlayerGlobals.musicChannel(MH_SONG2)
		PhasePlayerGlobals.player.musicPlaying = 0
		system_virtualTime = 0
		
		PhasePlayer_LaunchApp
		
		
		PhasePlayerGlobals.initialized = 0
		PhaseMenuGlobals.titleZoom = 0
		PhaseMenuGlobals.titleStartZoom = system_time + .5
		PhaseMenuGlobals.waitTicks = system_time * 1000 + 500
		
		PhaseMenuGlobals.titleFade = 0
		PhaseMenuGlobals.destination = 0
		PhaseEditorGlobals.winActive(WIN_MAIN) = 1
		glLineWidth 1
		
		for i = 0 to MH_SONG2 - 1
			fsound_stopsound PhasePlayerGlobals.musicChannel(i)
		next
		
		if PhaseMenuGlobals.winOptions.sfxOn = 1 then 
			PhasePlayerGlobals.musicChannel(MH_SUB_DROP) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_SUB_DROP))
			FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_SUB_DROP), PhaseMenuGlobals.winOptions.sfxVol
		end if
		
		PhasePlayer_Explosions_Clear_Explosions
		PhasePlayer_Bullets_Clear_Bullets
		PhasePlayer_Missiless_Clear_Missiles
		
		
	end if
	
	if PhaseMenuGlobals.destination = D_END and PhaseMenuGlobals.titleFade = 1 and PhaseMenuGlobals.titleZoom = 0 then PhaseMenuGlobals.exitCode = 1

end sub



'-------------------------------------------------------------
' Render - root procedure for all module drawing during the
'          main loop
'-------------------------------------------------------------
sub PhaseMenu_Render ()

	glClear  GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT

	glLoadIdentity
	
	glPushMatrix
	
	glDisable GL_CULL_FACE
	glDepthMask GL_FALSE
	glDisable GL_LIGHTING
	
	if PhaseMenuGlobals.titleStartZoom < system_time then
		
		
	glPushMatrix
		glTranslatef 0, 0, -50
		
		glRotatef PhaseMenuGlobals.gradSlider(0).angle.x, 1, 0, 0
		glRotatef PhaseMenuGlobals.gradSlider(0).angle.y, 0, 1, 0
		glRotatef PhaseMenuGlobals.gradSlider(0).angle.z, 0, 0, 1
		
		dim as integer i
		dim as single w0, posOfs
		
		w0 = 16
		posOfs = 10
		
		'----- draw scene
		
		for i = 0 to PhaseMenuGlobals.nSliders - 1
		
			glPushMatrix
				glTranslatef PhaseMenuGlobals.gradSlider(i).position.x * posOfs, PhaseMenuGlobals.gradSlider(i).position.y * posOfs, PhaseMenuGlobals.gradSlider(i).position.z * posOfs
				glRotatef PhaseMenuGlobals.gradSlider(i).angle.x, 1, 0, 0
				glRotatef PhaseMenuGlobals.gradSlider(i).angle.y, 0, 1, 0
				glRotatef PhaseMenuGlobals.gradSlider(i).angle.z, 0, 0, 1
				
				PhaseMenuGlobals.gradSlider(i).angle.x += PhaseMenuGlobals.gradSlider(i).velocityMag * 12 * PhaseMenuGlobals.gradSlider(i).velocity.x * system_ticksPassed / 1000
				PhaseMenuGlobals.gradSlider(i).angle.y += PhaseMenuGlobals.gradSlider(i).velocityMag * 12 * PhaseMenuGlobals.gradSlider(i).velocity.y * system_ticksPassed / 1000
				PhaseMenuGlobals.gradSlider(i).angle.z += PhaseMenuGlobals.gradSlider(i).velocityMag * 12 * PhaseMenuGlobals.gradSlider(i).velocity.z * system_ticksPassed / 1000
				
				glBegin GL_QUADS
				
					glColor4f 1, 1, 1, 0
					
					glVertex2f -w0, w0
					glVertex2f -w0, -w0
					
					glColor4f 1, 1, 1, PhaseMenuGlobals.titleZoom
					glVertex2f w0, -w0
					glVertex2f w0, w0
					
				glEnd
			glPopMatrix
			
		next
		
	glPopMatrix
	
	
		if PhaseMenuGlobals.titleFade = 0 then 
			if PhaseMenuGlobals.titleZoom < 1 then PhaseMenuGlobals.titleZoom += (2 * system_ticksPassed / 1000) * (1 - PhaseMenuGlobals.titleZoom)
			if PhaseMenuGlobals.titleZoom > .9999 then PhaseMenuGlobals.titleZoom = 1
		else
			if PhaseMenuGlobals.titleZoom > 0 then PhaseMenuGlobals.titleZoom -= (2 * system_ticksPassed / 1000) * (PhaseMenuGlobals.titleZoom)
			if PhaseMenuGlobals.titleZoom < .01 then PhaseMenuGlobals.titleZoom = 0
		end if
		
		PhaseMenuGlobals.titleAngle += tempx * system_ticksPassed / 1000
		
		dim as integer nStp
		dim as single an0, xOfs, yOfs
		
		
		glPushMatrix
			glTranslatef 0, 1.28 * PhaseMenuGlobals.titleZoom, -18 * PhaseMenuGlobals.titleZoom
			
			
			glColor4f 0, 0, 0, PhaseMenuGlobals.titleZoom
			Render_Model_Plain ModelDB[PhaseMenuGlobals.titleMID].modelHandle
			
			
			glColor4f 0, 0, 0, .077 * PhaseMenuGlobals.titleZoom
			
			nStp = 20
			for i = 0 to nStp - 1
			
				an0 = i / nStp
				xOfs = .028 * cos(3.1415 * 2 * an0)
				yOfs = .028 * sin(3.1415 * 2 * an0)
				
				glPushmatrix
					glTranslatef xOfs, yOfs, 0
					Render_Model_Plain ModelDB[PhaseMenuGlobals.titleMID].modelHandle
				glPopMatrix
			
			next
			
			
		glPopMatrix
		
	end if
	
	
	glEnable GL_CULL_FACE
	glDepthMask GL_TRUE
	glEnable GL_LIGHTING
	
	'----- end drawing
	
	
	
	glPopMatrix
	glFlush


end sub



sub PhaseMenu_Render_Gui_Layer ()


	system_2dplane_set_full
	'---- draw gui elements
	
	PhaseMenu_Render_Gui_Main
	PhaseMenu_Render_Gui_Options
	PhaseMenu_Render_Gui_Credits
	PhaseMenu_Render_Gui_Controls
	
	PhaseMenu_Render_Gui_HighScores
	PhaseMenu_Render_Gui_HSData
	PhaseMenu_Render_Gui_NewHighScore
	
	PhaseEditor_Render_Gui_SelectColor
	PhaseEditor_Render_Gui_SelectTable_Game
	PhaseEditor_Render_Gui_TextBoxWindow
	PhaseEditor_Render_Gui_NumberBoxWindow
	PhaseEditor_Render_Gui_CoorBoxWindow
	
	'---- end drawing	
	system_2dplane_clear


end sub



'-------------------------------------------------------------
' Shutdown - called before exiting module to clear any initial
'            conditions that could affect other modules
'-------------------------------------------------------------
sub PhaseMenu_Shutdown ()

	dim as integer i
	
	for i = 0 to MH_SONG2
		fsound_sample_Free PhasePlayerGlobals.musicHandle(i)
		fsound_stopsound PhasePlayerGlobals.musicChannel(i)
	next
	
	fsound_close
end sub
