

'--------------------
'  Globals - globals for this module are stored in this type then shared
'--------------------

'-------------------------------------------------------------
' DeInit - include call inside Initialize.bas(DeInit)
'		   if you need to unload memory before program exits
'-------------------------------------------------------------
sub PhasePause_DeInit ()
end sub



'-------------------------------------------------------------
' Initialize - load objects specific to this module in memory
'-------------------------------------------------------------
sub PhasePause_Initialize ()

	PhasePauseGlobals.initialized = 1
	PhasePauseGlobals.exitCode = 0
	
	dim as integer i
	
	PhasePauseGlobals.nSliders = 20
	
	for i = 0 to PhasePauseGlobals.nSliders - 1
		PhasePauseGlobals.gradSlider(i).position.x = rnd * 10 - 5
		PhasePauseGlobals.gradSlider(i).position.y = rnd * 10 - 5
		PhasePauseGlobals.gradSlider(i).position.z = rnd * 10 - 5
		
		math_normalize3f PhasePauseGlobals.gradSlider(i).position
		
		PhasePauseGlobals.gradSlider(i).angle.x = rnd * 360
		PhasePauseGlobals.gradSlider(i).angle.y = rnd * 360
		PhasePauseGlobals.gradSlider(i).angle.z = rnd * 360
		
		PhasePauseGlobals.gradSlider(i).velocity.x = rnd * 10 - 5
		PhasePauseGlobals.gradSlider(i).velocity.y = rnd * 10 - 5
		PhasePauseGlobals.gradSlider(i).velocity.z = rnd * 10 - 5
		
		math_normalize3f PhasePauseGlobals.gradSlider(i).velocity
		
		PhasePauseGlobals.gradSlider(i).velocityMag = rnd * 2 - 1
		
		
	next
	
	PhasePauseGlobals.titleMID = objFile_load ("data/title.obj")
	PhasePauseGlobals.pausedMID = objFile_load ("data/paused.obj")
	
end sub



'-------------------------------------------------------------
' LaunchApp - used to call the module. on first run will call
'		      Initialize(). then setup module initial operating
'				  conditions
'-------------------------------------------------------------
sub PhasePause_LaunchApp ()

	if PhasePauseGlobals.initialized = 0 then PhasePause_Initialize

	PhaseEditorGlobals.winActive(WIN_PAUSED_MAIN) = 1
	PhaseEditorGlobals.winLock(WIN_PAUSED_MAIN) = 0
	
	
	PhasePauseGlobals.titleZoom = 0
	PhasePauseGlobals.titleZoomSpeed = 0
	PhasePauseGlobals.titleAngle = 0
	PhasePauseGlobals.titleStartZoom = system_time + .5
	PhasePauseGlobals.waitTicks = system_time * 1000 + 300
	
	dim as integer i
	
	PhasePlayerGlobals.player.musicPlaying = 0
	FSOUND_SetPaused PhasePlayerGlobals.musicChannel(MH_SONG), 1
	
	if PhaseMenuGlobals.winOptions.sfxOn = 1 then 
		PhasePlayerGlobals.musicChannel(MH_SUB_DROP) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_SUB_DROP))
		FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_SUB_DROP), PhaseMenuGlobals.winOptions.sfxVol
	end if
	
	PhasePauseGlobals.titleFade = 0
	PhasePauseGlobals.destination = 0
	
	PhasePause_MainLoop

	PhasePause_ShutDown

end sub



'-------------------------------------------------------------
' MainLoop - module program loop
'-------------------------------------------------------------
sub PhasePause_MainLoop ()


	do
		PhasePause_ProcessInputs
		
		PhasePause_Render
		if PhasePauseGlobals.titleZoom > .8 then PhasePause_Render_Gui_Layer
		
		glfwSwapBuffers
		
		system_upd_fpsr
		
		
	loop while PhasePauseGlobals.exitCode = 0


end sub



'-------------------------------------------------------------
' ProcessInputs - Performs input processing.
'-------------------------------------------------------------
sub PhasePause_ProcessInputs ()


	dim as integer i
	
	System_Process_Adjustment_Inputs
	
	
	'--- check mouse input
	glfwGetMousePos @PhaseEditorGlobals.mousex, @PhaseEditorGlobals.mousey
	
	
	'--- handle key input
	System_Process_Key_Strokes
	
	if PhaseMenuGlobals.winOptions.musicOn = 1 then 
		if PhasePlayerGlobals.player.musicPlaying = 0 then 
			PhasePlayerGlobals.musicChannel(MH_SONG2) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_SONG2))
			PhasePlayerGlobals.player.musicPlaying = 1
		end if
		FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_SONG2), PhaseMenuGlobals.winOptions.musicVol * PhasePauseGlobals.titleZoom
	end if
	
	if PhasePauseGlobals.waitTicks > system_ticks then exit sub
	
	if PhasePauseGlobals.destination = D_GAME and PhasePauseGlobals.titleFade = 1 and PhasePauseGlobals.titleZoom = 0 then
		PhasePauseGlobals.exitCode = 1
		
		fsound_stopsound PhasePlayerGlobals.musicChannel(MH_SONG2)
		FSOUND_SetPaused PhasePlayerGlobals.musicChannel(MH_SONG), 0
	end if
	
	
	if PhasePauseGlobals.destination = D_END and PhasePauseGlobals.titleFade = 1 and PhasePauseGlobals.titleZoom = 0 then
		PhasePauseGlobals.exitCode = 1
		
		PhasePlayerGlobals.exitCode = 1
		PhaseEditorGlobals.winActive(WIN_MAIN) = 1
	end if
	
	
	if PhasePauseGlobals.destination = D_EXIT and PhasePauseGlobals.titleFade = 1 and PhasePauseGlobals.titleZoom = 0 then
		PhasePauseGlobals.exitCode = 1
		PhaseMenuGlobals.exitCode = 1
		PhasePlayerGlobals.exitCode = 1
		PhaseEditorGlobals.winActive(WIN_MAIN) = 1
	end if
	
	
end sub



'-------------------------------------------------------------
' Render - root procedure for all module drawing during the
'          main loop
'-------------------------------------------------------------
sub PhasePause_Render ()

	glClear  GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT

	glLoadIdentity
	
	glPushMatrix
	
	glDisable GL_CULL_FACE
	glDepthMask GL_FALSE
	glDisable GL_LIGHTING
	
	if PhasePauseGlobals.titleStartZoom < system_time then
		
		
	glPushMatrix
		glTranslatef 0, 0, -50
		
		glRotatef PhasePauseGlobals.gradSlider(0).angle.x, 1, 0, 0
		glRotatef PhasePauseGlobals.gradSlider(0).angle.y, 0, 1, 0
		glRotatef PhasePauseGlobals.gradSlider(0).angle.z, 0, 0, 1
		
		dim as integer i
		dim as single w0, posOfs
		
		w0 = 16
		posOfs = 10
		
		'----- draw scene
		
		for i = 0 to PhasePauseGlobals.nSliders - 1
		
			glPushMatrix
				glTranslatef PhasePauseGlobals.gradSlider(i).position.x * posOfs, PhasePauseGlobals.gradSlider(i).position.y * posOfs, PhasePauseGlobals.gradSlider(i).position.z * posOfs
				glRotatef PhasePauseGlobals.gradSlider(i).angle.x, 1, 0, 0
				glRotatef PhasePauseGlobals.gradSlider(i).angle.y, 0, 1, 0
				glRotatef PhasePauseGlobals.gradSlider(i).angle.z, 0, 0, 1
				
				PhasePauseGlobals.gradSlider(i).angle.x += PhasePauseGlobals.gradSlider(i).velocityMag * 12 * PhasePauseGlobals.gradSlider(i).velocity.x * system_ticksPassed / 1000
				PhasePauseGlobals.gradSlider(i).angle.y += PhasePauseGlobals.gradSlider(i).velocityMag * 12 * PhasePauseGlobals.gradSlider(i).velocity.y * system_ticksPassed / 1000
				PhasePauseGlobals.gradSlider(i).angle.z += PhasePauseGlobals.gradSlider(i).velocityMag * 12 * PhasePauseGlobals.gradSlider(i).velocity.z * system_ticksPassed / 1000
				
				glBegin GL_QUADS
				
					glColor4f 1, 1, 1, 0
					
					glVertex2f -w0, w0
					glVertex2f -w0, -w0
					
					glColor4f 1, 1, 1, PhasePauseGlobals.titleZoom
					glVertex2f w0, -w0
					glVertex2f w0, w0
					
				glEnd
			glPopMatrix
			
		next
		
	glPopMatrix
	
	
		if PhasePauseGlobals.titleFade = 0 then 
			if PhasePauseGlobals.titleZoom < 1 then PhasePauseGlobals.titleZoom += (2 * system_ticksPassed / 1000) * (1 - PhasePauseGlobals.titleZoom)
			if PhasePauseGlobals.titleZoom > .9999 then PhasePauseGlobals.titleZoom = 1
		else
			if PhasePauseGlobals.titleZoom > 0 then PhasePauseGlobals.titleZoom -= (2 * system_ticksPassed / 1000) * (PhasePauseGlobals.titleZoom)
			if PhasePauseGlobals.titleZoom < .01 then PhasePauseGlobals.titleZoom = 0
		end if
		
		PhasePauseGlobals.titleAngle += tempx * system_ticksPassed / 1000
		
		dim as integer nStp
		dim as single an0, xOfs, yOfs
		
		
		glPushMatrix
			glTranslatef 0, 1.28 * PhasePauseGlobals.titleZoom, -18 * PhasePauseGlobals.titleZoom
			
			
			glColor4f 0, 0, 0, PhasePauseGlobals.titleZoom
			Render_Model_Plain ModelDB[PhasePauseGlobals.titleMID].modelHandle
			
			
			
			glColor4f 0, 0, 0, .077 * PhasePauseGlobals.titleZoom
			
			nStp = 20
			for i = 0 to nStp - 1
			
				an0 = i / nStp
				xOfs = .028 * cos(3.1415 * 2 * an0)
				yOfs = .028 * sin(3.1415 * 2 * an0)
				
				glPushmatrix
					glTranslatef xOfs, yOfs, 0
					Render_Model_Plain ModelDB[PhasePauseGlobals.titleMID].modelHandle
				glPopMatrix
			
			next
			
			glTranslatef 0, -1, -8.4
			
			glColor4f 0, 0, 0, PhasePauseGlobals.titleZoom
			Render_Model_Plain ModelDB[PhasePauseGlobals.pausedMID].modelHandle
			
			
			
			glColor4f 0, 0, 0, .077 * PhasePauseGlobals.titleZoom
			
			nStp = 20
			for i = 0 to nStp - 1
			
				an0 = i / nStp
				xOfs = .028 * cos(3.1415 * 2 * an0)
				yOfs = .028 * sin(3.1415 * 2 * an0)
				
				glPushmatrix
					glTranslatef xOfs, yOfs, 0
					Render_Model_Plain ModelDB[PhasePauseGlobals.pausedMID].modelHandle
				glPopMatrix
			
			next
			
			
		glPopMatrix
		
	end if
	
	
	glEnable GL_CULL_FACE
	glDepthMask GL_TRUE
	glEnable GL_LIGHTING
	
	'----- end drawing
	
	
	
	glPopMatrix
	glFlush


end sub



sub PhasePause_Render_Gui_Layer ()


	system_2dplane_set_full
	'---- draw gui elements
	
	PhasePause_Render_Gui_Main
	
	'---- end drawing	
	system_2dplane_clear


end sub



'-------------------------------------------------------------
' Shutdown - called before exiting module to clear any initial
'            conditions that could affect other modules
'-------------------------------------------------------------
sub PhasePause_Shutdown ()

	dim as integer i
	
	fsound_stopsound PhasePlayerGlobals.musicChannel(MH_SONG2)

end sub
