DECLARE sub PhasePlayer_DeInit ()
DECLARE sub PhasePlayer_Initialize ()
DECLARE sub PhasePlayer_LaunchApp ()
DECLARE sub PhasePlayer_MainLoop ()
DECLARE sub PhasePlayer_ProcessInputs ()
DECLARE sub PhasePlayer_Render ()
DECLARE sub PhasePlayer_Shutdown ()


'--------------------
'  Globals - globals for this module are stored in this type then shared
'--------------------

type PhasePlayerGlobalsType


	initialized		as byte
	exitCode		as byte

	nMissiles		as integer
	lastMissile		as integer
	missile			as MissileType ptr
	
	nWeapons		as integer
	weaponClass		as WeaponClassType ptr
	
	nMissileClasses	as integer
	missileClass	as MissileClassType ptr
	
	nExplosions		as integer
	lastExplosion	as integer
	explosion		as ExplosionType ptr
	
	nExplosionClasses	as integer
	explosionClass	as ExplosionClassType ptr
	
	nBullets		as integer
	lastBullet		as integer
	bullet			as BulletType ptr
	
	nBulletClasses	as integer
	bulletClass		as BulletClassType ptr
	
	player			as PlayerType
	nEnemies		as integer
	enemy			as EnemyType ptr
	
	cameraPosition	as Vector3D
	waitKey			as integer
	fuzzMode		as integer
	
	cameraAngle		as Vector3D
	
	missionClock		as clockdat
	missionTotalTime	as single
	
	musicEnabled		as integer
	musicHandle(16)		as FSOUND_SAMPLE ptr
	musicChannel(16)	as integer

	powerUp			as PowerUpType
	
	sevenSegmentMID(10)		as integer
	sevenSegmentDL(10)		as integer
	missionClockBGDL		as integer
	
	chromeTex		as integer
	
	waitTicks		as integer
	system_ticks	as integer
	mousex			as integer
	mousey			as integer
	
end type

#DEFINE MH_SONG		0
#DEFINE MH_GUNS		1
#DEFINE MH_SUB_DROP	2
#DEFINE MH_COCK		3
#DEFINE MH_HIT		4
#DEFINE MH_RICOCHET 5
#DEFINE MH_POWERUP	6
#DEFINE MH_NOAMMO	7
#DEFINE MH_ALARM	8
#DEFINE MH_SONG2	9

dim shared PhasePlayerGlobals as PhasePlayerGlobalsType


'-------------------------------------------------------------
' DeInit - include call inside Initialize.bas(DeInit)
'		   if you need to unload memory before program exits
'-------------------------------------------------------------
sub PhasePlayer_DeInit ()
end sub



'-------------------------------------------------------------
' Initialize - load objects specific to this module in memory
'-------------------------------------------------------------
sub PhasePlayer_Initialize ()

	PhasePlayerGlobals.initialized = 1
	PhasePlayerGlobals.exitCode = 0
	
	PhasePlayer_Init_Player
	
	PhaseData_PhaseSequence_Load "0.seq"
	
	PhaseDataGlobals.phaseSequence.startTime = system_time + 10
	PhasePlayer_Init_MissionClock

	
	glfwSetMousePos OpenGLProperties.SCR_WIDTH / 2, OpenGLProperties.SCR_HEIGHT / 2
	glLineWidth OpenGLProperties.SCR_WIDTH / 640
	
	system_nextticks = system_time * 1000 + 1000
	PhasePlayerGlobals.player.lockCamera = system_nextTicks - 800
	PhasePlayerGlobals.player.fadeInTime = system_nextticks
	PhasePlayerGlobals.waitKey = 0
	
end sub



'-------------------------------------------------------------
' LaunchApp - used to call the module. on first run will call
'		      Initialize(). then setup module initial operating
'				  conditions
'-------------------------------------------------------------
sub PhasePlayer_LaunchApp ()

	if PhasePlayerGlobals.initialized = 0 then PhasePlayer_Initialize



	PhasePlayer_MainLoop

	PhasePlayer_ShutDown

end sub



'-------------------------------------------------------------
' MainLoop - module program loop
'-------------------------------------------------------------
sub PhasePlayer_MainLoop ()


	do
		
		PhasePlayer_ProcessInputs
		PhasePlayer_Render
		
		PhasePlayer_Player_Upd_Player
		PhasePlayer_Bullets_upd_Bullets
		PhasePlayer_Explosions_upd_Explosions
		PhasePlayer_Missiles_upd_Missiles
		PhasePlayer_Projectiles_Upd_Projectiles
		PhasePlayer_Waves_Upd_Waves
		
		glfwSwapBuffers
		
		system_upd_fpsr
		
	loop while PhasePlayerGlobals.exitCode = 0


end sub



'-------------------------------------------------------------
' ProcessInputs - Performs input processing.
'-------------------------------------------------------------
sub PhasePlayer_ProcessInputs ()

	dim as integer a, b, i
	
	if PhasePlayerGlobals.waitTicks < system_ticks then
	
		if glfwGetKey(GLFW_KEY_F12) then
			PhasePlayerGlobals.waitTicks = system_ticks + 300
			PhasePlayer_Player_PowerUp_Pickup int(rnd * 3)
		end if
		
		if glfwGetKey(GLFW_KEY_F11) then
			PhaseDataGlobals.phaseSequence.wave[PhaseDataGlobals.phaseSequence.nWaves - 2].startTime = 0
			for b = 0 to PhaseDataGlobals.phaseSequence.wave[PhaseDataGlobals.phaseSequence.nWaves - 2].nSpawn - 1
				PhaseDataGlobals.phaseSequence.wave[PhaseDataGlobals.phaseSequence.nWaves - 2].spawn[b].spawnTime = -1
			next
			PhasePlayerGlobals.waitTicks = system_ticks + 300
		end if
		
	end if
	
	if glfwGetKey(GLFW_KEY_ESC) and PhasePlayerGlobals.waitTicks < system_ticks then
		
		PhasePlayerGlobals.system_ticks = system_ticks
		PhasePauseGlobals.exitCode = 0
		io_clock_pause  PhasePlayerGlobals.missionClock
		
		
		for i = 0 to MH_SONG2 - 1
			fsound_stopsound PhasePlayerGlobals.musicChannel(i)
		next
		
		PhasePlayerGlobals.player.fireSoundLock = 0
		PhasePlayerGlobals.player.ricochetLock = 0
		PhasePlayerGlobals.player.alarmLock = 0
		
		
		glfwGetMousePos @PhasePlayerGlobals.mousex, @PhasePlayerGlobals.mousex
		glfwEnable GLFW_MOUSE_CURSOR
		
		PhasePause_LaunchApp
		
		PhasePlayerGlobals.player.musicPlaying = 0
		
		glfwDisable GLFW_MOUSE_CURSOR
		
		glfwSetMousePos PhasePlayerGlobals.mousex, PhasePlayerGlobals.mousex
		system_virtualTime += (system_ticks - PhasePlayerGlobals.system_ticks) / 1000
		
		for a = 0 to PhaseDataGlobals.phaseSequence.nWaves - 1
			for b = 0 to PhaseDataGlobals.phaseSequence.wave[a].nSpawn - 1
				if PhaseDataGlobals.phaseSequence.wave[a].spawn[b].active = 0 then PhaseDataGlobals.phaseSequence.wave[a].spawn[b].spawnTime = -1
			next
		next
		
		system_ticks = system_time * 1000
		system_upd_fpsr
		system_ticksPassed = 0
		PhasePlayerGlobals.waitTicks = system_ticks + 300
		
	end if

	System_Process_Adjustment_Inputs

	if PhasePlayerGlobals.player.lockCamera < system_ticks then
		if glfwGetKey(GLFW_KEY_SPACE) or glfwGetMouseButton(GLFW_MOUSE_BUTTON_LEFT) = GLFW_PRESS then PhasePlayer_Player_FireWeapon
	end if
	
	if PhaseMenuGlobals.winOptions.sfxOn = 1 then
	if PhasePlayerGlobals.player.fireSoundEnd > system_time and PhasePlayerGlobals.player.selWeapon = 0 then
		if PhasePlayerGlobals.player.fireSoundLock = 0 then
			PhasePlayerGlobals.musicChannel(MH_GUNS) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_GUNS))
			PhasePlayerGlobals.player.fireSoundLock = 1
			PhasePlayerGlobals.player.fireSoundVol = PhaseMenuGlobals.winOptions.sfxVol
			FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_GUNS), PhasePlayerGlobals.player.fireSoundVol
		end if
		
	end if
	
	if PhasePlayerGlobals.player.ricochetEnd > system_time then
		if PhasePlayerGlobals.player.ricochetLock = 0 then
			PhasePlayerGlobals.musicChannel(MH_RICOCHET) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_RICOCHET))
			PhasePlayerGlobals.player.ricochetLock = 1
			PhasePlayerGlobals.player.ricochetVol = PhaseMenuGlobals.winOptions.sfxVol
			FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_RICOCHET), PhasePlayerGlobals.player.ricochetVol
		end if
		
	end if
	
	if PhasePlayerGlobals.player.health / PhasePlayerGlobals.player.maxHealth < .4 then
		PhasePlayerGlobals.player.alarmEnd = (system_ticks / 1000) + .5
		if PhasePlayerGlobals.player.alarmLock = 0 then
			PhasePlayerGlobals.musicChannel(MH_ALARM) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_ALARM))
			PhasePlayerGlobals.player.alarmLock = 1
			FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_ALARM), PhaseMenuGlobals.winOptions.sfxVol
		end if
	end if
	end if
	
	if system_time > PhasePlayerGlobals.player.alarmEnd and PhasePlayerGlobals.player.alarmLock = 1 then
		Fsound_stopsound PhasePlayerGlobals.musicChannel(MH_ALARM)
		PhasePlayerGlobals.player.alarmLock = 0
	end if
	
	if system_time > PhasePlayerGlobals.player.fireSoundEnd and PhasePlayerGlobals.player.fireSoundLock = 1 then
		Fsound_stopsound PhasePlayerGlobals.musicChannel(MH_GUNS)
		PhasePlayerGlobals.player.fireSoundLock = 0
	end if
	
	if system_time > PhasePlayerGlobals.player.ricochetEnd and PhasePlayerGlobals.player.ricochetLock = 1 then
		Fsound_stopsound PhasePlayerGlobals.musicChannel(MH_RICOCHET)
		PhasePlayerGlobals.player.ricochetLock = 0
	end if
	
	
	if glfwGetKey(GLFW_KEY_F1) and PhasePlayerGlobals.waitKey < system_ticks then
		PhasePlayerGlobals.waitKey = system_ticks + 200
		PhasePlayerGlobals.fuzzMode = 1 - PhasePlayerGlobals.fuzzMode
	end if
	
	dim as integer dMouseX, dMouseY
	
	
	if glfwGetWindowParam(GLFW_ACTIVE) then
	
		if PhasePlayerGlobals.player.lockCamera > system_ticks then
			glfwSetMousePos OpenGLProperties.SCR_WIDTH / 2, OpenGLProperties.SCR_HEIGHT / 2
			PhasePlayerGlobals.cameraAngle.x = 0
			PhasePlayerGlobals.cameraAngle.y = 0
			PhasePlayerGlobals.player.targetCameraX = PhasePlayerGlobals.cameraAngle.x
			PhasePlayerGlobals.player.targetCameraY = PhasePlayerGlobals.cameraAngle.y
			PhasePlayerGlobals.player.hudCameraX = PhasePlayerGlobals.player.targetCameraX
			PhasePlayerGlobals.player.hudCameraY = PhasePlayerGlobals.player.targetCameraY
		end if
		
		glfwGetMousePos @dMouseX, @dMouseY
		dMouseX -= OpenGLProperties.SCR_WIDTH / 2
		dMouseY -= OpenGLProperties.SCR_HEIGHT / 2
		
		PhasePlayerGlobals.cameraAngle.x -= dMouseX * .005 * system_ticksPassed * PhaseMenuGlobals.winOptions.sensitivity / 128
		if PhaseMenuGlobals.winOptions.invertMouse = 0 then PhasePlayerGlobals.cameraAngle.y -= dMouseY * .005 * system_ticksPassed * PhaseMenuGlobals.winOptions.sensitivity / 128
		if PhaseMenuGlobals.winOptions.invertMouse = 1 then PhasePlayerGlobals.cameraAngle.y += dMouseY * .005 * system_ticksPassed * PhaseMenuGlobals.winOptions.sensitivity / 128
		
		PhasePlayerGlobals.player.targetCameraX = PhasePlayerGlobals.cameraAngle.x
		PhasePlayerGlobals.player.targetCameraY = PhasePlayerGlobals.cameraAngle.y
		
		glfwSetMousePos OpenGLProperties.SCR_WIDTH / 2, OpenGLProperties.SCR_HEIGHT / 2
		glfwDisable GLFW_MOUSE_CURSOR
	else
		glfwEnable GLFW_MOUSE_CURSOR
	end if
	
	if glfwGetMouseButton(GLFW_MOUSE_BUTTON_RIGHT) and PhasePlayerGlobals.waitKey < system_ticks then
		PhasePlayerGlobals.waitKey = system_ticks + 250
		PhasePlayerGlobals.player.selWeapon += 1
		if PhasePlayerGlobals.player.selWeapon = PhasePlayerGlobals.player.nWeapons then PhasePlayerGlobals.player.selWeapon = 0
		PhasePlayerGlobals.player.flashMissileRed = 1
		
		if PhaseMenuGlobals.winOptions.sfxOn = 1 then 
			PhasePlayerGlobals.musicChannel(MH_COCK) = FSOUND_PlaySound(FSOUND_FREE, PhasePlayerGlobals.musicHandle(MH_COCK))
			FSOUND_setVolume PhasePlayerGlobals.musicChannel(MH_COCK), PhaseMenuGlobals.winOptions.sfxVol
		end if
	end if
	
	'---update clock
	io_clock_update PhasePlayerGlobals.missionClock
	
	
end sub



'-------------------------------------------------------------
' Render - root procedure for all module drawing during the
'          main loop
'-------------------------------------------------------------
sub PhasePlayer_Render ()


	glClear  GL_COLOR_BUFFER_BIT OR GL_DEPTH_BUFFER_BIT

	glLoadIdentity
	glPushMatrix
	
	
	
	'----- draw scene
	
	PhasePlayerGlobals.fuzzMode = 1
	if PhasePlayerGlobals.player.fuzzTime < system_ticks then PhasePlayerGlobals.fuzzMode = 0
	
	if PhasePlayerGlobals.fuzzMode = 0 then PhasePlayer_Render_Scene
	if PhasePlayerGlobals.fuzzMode = 1 then PhasePlayer_Render_Scene_Fuzz
	PhasePlayer_Render_HUD
	PhasePlayer_Render_HUD_Clock
	
	
	'----- end drawing
	
	
	glPopMatrix
	glFlush


end sub



'-------------------------------------------------------------
' Shutdown - called before exiting module to clear any initial
'            conditions that could affect other modules
'-------------------------------------------------------------
sub PhasePlayer_Shutdown ()

	glfwEnable GLFW_MOUSE_CURSOR

	PhasePlayerGlobals.player.alarmLock = 0
	PhasePlayerGlobals.player.fireSoundLock = 0
	PhasePlayerGlobals.player.ricochetLock = 0
	PhasePlayerGlobals.player.musicPlaying = 0
end sub
