type HighScoreType

	name			as string
	points			as integer
	kills			as single
	totalSpawn		as single
	damage			as single
	totalHealth		as single
	mapDestroyed	as single
	totalProjectiles	as single
	powerUps		as single
	totalPowerUps	as single
	accuracy		as single
	healthLost		as single
	shieldLost		as single
	timeBonus		as single
	
end type
	

type ScoreDataType

	value			as single
	frequency		as single
	weight			as integer

end type
	

type ScoreType

	kills			as ScoreDataType
	damage			as ScoreDataType
	mapDestroyed	as ScoreDataType
	timeRemaining	as ScoreDataType
	powerUps		as ScoreDataType
	healthLost		as ScoreDataType
	shieldLost		as ScoreDataType
	accuracy		as ScoreDataType
	
	shotsFired		as integer
	shotsLanded		as integer
	totalFreq		as single
	points			as integer
	timeBonus		as single

end type


type PowerUpType
	nextSpawn		as single
	active			as integer
	class			as integer
	position		as Vector3D
	matrix(16)		as single
end type


type BulletClassType
	title			as string
end type


type BulletType
	active			as integer
	class			as integer
	pattern			as integer
	position		as vector3d
	velocity		as vector3d
	angle			as vector3d
	matrix(16)		as single
	nGenerators		as integer
	generator		as GeneratorClassType ptr
	homeTime		as integer
end type


type EnemyType

	active			as integer
	class			as integer
	health			as single
	lastFire		as integer
	
	
end type


type BlastRingType
	startTime		as single
	duration		as single
	radius			as single
end type


type MetaBallType
	startTime		as single
	duration		as single
	colorCenter		as vector3D
	angle			as single
	baseColorOffset	as single
	radius			as single
end type


type ExplosionType
	active			as integer		'when no longer active, deallocate particle and metaball
	class			as integer
	position		as Vector3D
	
	startTime		as integer
	
	angle			as vector3d
	matrix(16)		as single
	nGenerators		as integer
	generator		as GeneratorClassType ptr
	
	nMetaBalls		as integer
	metaBall		as MetaBallType ptr
	
	nBlastRings		as integer
	blastRing		as BlastRingType ptr
end type


type ExplosionClassType
	fallOff			as single
	
	nParticles		as integer		'+- 20%
	particleSize	as single
	
	nSparks			as integer		'+- 20%
	sparkColorA		as rgbAlpha
	sparkColorB		as rgbAlpha
	
	nMetaBalls		as integer		'+- 20%
	metaBallColorA	as rgbAlpha
	metaBallColorB	as rgbAlpha
	metaBallColorC	as rgbAlpha
end type


type MissileClassType

	mass			as integer
	
end type


type MissileType

	active			as integer
	class			as integer
	owner			as integer		'0 = player
	position		as vector3D
	nextposition	as vector3D
	velocity		as Vector3D
	accelVector		as Vector3D
	accelMag		as single
	mass			as single
	matrix(16)		as single
	impact			as integer
	explosionClass	as integer

end type


type WeaponClassType
	
	reloadTime		as integer
	rpm				as integer		'rounds per minute. => rounds per 60,000 ticks
	missileClass	as integer
	jutter			as single

end type


type WeaponType

	class			as integer
	rounds			as integer
	lastFire		as integer
	angle			as single

end type


type PlayerType
	
	nWeapons		as integer
	selWeapon		as integer
	weapon			as WeaponType ptr
	fuzzTime		as integer
	
	health			as single
	maxHealth		as single
	shield			as single
	maxShield		as single
	
	flashShield		as single
	flashMissile	as single
	flashTime		as single
	flashHealth		as single
	flashShieldRed	as single
	flashHealthRed	as single
	flashMissileRed	as single
	
	
	fireSoundOn		as integer
	fireSoundVol	as single
	fireSoundLock	as integer
	fireSoundEnd	as single
	
	musicPlaying	as integer
	
	ricochetOn		as integer
	ricochetLock	as integer
	ricochetEnd		as single
	play_ricochet	as integer
	ricochetVol		as single
	
	alarmOn			as integer
	alarmLock		as integer
	alarmEnd		as single
	
	hoverSoundLock	as integer
	
	hudCameraX		as single
	hudCameraY		as single
	targetCameraX	as single
	targetCameraY	as single
	lockCamera		as integer
	
	fadeInTime		as integer
	
	gameOver				as integer
	gameOverFadeBlackTime	as single
	gameOverFadeRed			as single
	gameoverExplosionTime	as single
	
	score			as scoreType
	highScores(20)	as HighScoreType
	
end type


#DEFINE MC_BULLET			0
#DEFINE MC_ROCKET			1

#DEFINE WC_MACHINE_GUN		0
#DEFINE WC_ROCKET_LAUNCHER	1

#DEFINE EC_NORMAL			0
#DEFINE EC_SHRAPNULL		1
#DEFINE EC_SMALL			2


'---- bullet patterns
#DEFINE BP_ONE_SHOT				0
#DEFINE BP_BOMB					1
#DEFINE BP_MULT_SHOT_AND_BOMB	2

'---- bullet classes
#DEFINE BC_TYPE_I				0


'---- power up classes
#DEFINE PU_HEALTH			0
#DEFINE PU_ARMOUR			1
#DEFINE PU_AMMO				2